title: The reactive manifesto
url: http://www.reactivemanifesto.org/
hash_url: 00c9fbe2bf8fb00270b36167878d952a

<article>

<section id="the-reactive-manifesto">

    <p>Organisations working in disparate domains are independently discovering patterns for building software that look the same. These systems are more robust, more resilient, more flexible and better positioned to meet modern demands.</p>
    <p>These changes are happening because application requirements have changed dramatically in recent years. Only a few years ago a large application had tens of servers, seconds of response time, hours of offline maintenance and gigabytes of data. Today applications are deployed on everything from mobile devices to cloud-based clusters running thousands of multi-core processors. Users expect millisecond response times and 100% uptime. Data is measured in Petabytes. Today's demands are simply not met by yesterday’s software architectures.</p>
    <p>We believe that a coherent approach to systems architecture is needed, and we believe that all necessary aspects are already recognised individually: we want systems that are Responsive, Resilient, Elastic and Message Driven. We call these Reactive Systems.</p>
    <p>Systems built as Reactive Systems are more flexible, loosely-coupled and <a href="http://www.reactivemanifesto.org/glossary#Scalability">scalable</a>. This makes them easier to develop and amenable to change. They are significantly more tolerant of failure and when <a href="http://www.reactivemanifesto.org/glossary#Failure">failure</a> does occur they meet it with elegance rather than disaster. Reactive Systems are highly responsive, giving <a href="http://www.reactivemanifesto.org/glossary#User">users</a> effective interactive feedback.</p>
    <h3>Reactive Systems are:</h3>

</section>

<section id="responsive" class="trait">

    <p><strong>Responsive: </strong>The <a href="http://www.reactivemanifesto.org/glossary#System">system</a> responds in a timely manner if at all possible. Responsiveness is the cornerstone of usability and utility, but more than that, responsiveness means that problems may be detected quickly and dealt with effectively. Responsive systems focus on providing rapid and consistent response times, establishing reliable upper bounds so they deliver a consistent quality of service. This consistent behaviour in turn simplifies error handling, builds end user confidence, and encourages further interaction.</p>

</section>

<section id="resilient" class="trait">

    <p><strong>Resilient: </strong>The system stays responsive in the face of <a href="http://www.reactivemanifesto.org/glossary#Failure">failure</a>. This applies not only to highly-available, mission critical systems — any system that is not resilient will be unresponsive after a failure. Resilience is achieved by <a href="http://www.reactivemanifesto.org/glossary#Replication">replication</a>, containment, <a href="http://www.reactivemanifesto.org/glossary#Isolation">isolation</a> and <a href="http://www.reactivemanifesto.org/glossary#Delegation">delegation</a>. Failures are contained within each <a href="http://www.reactivemanifesto.org/glossary#Component">component</a>, isolating components from each other and thereby ensuring that parts of the system can fail and recover without compromising the system as a whole. Recovery of each component is delegated to another (external) component and high-availability is ensured by replication where necessary. The client of a component is not burdened with handling its failures.</p>

</section>

<section id="elastic" class="trait">

    <p><strong>Elastic: </strong>The system stays responsive under varying workload. Reactive Systems can react to changes in the input rate by increasing or decreasing the <a href="http://www.reactivemanifesto.org/glossary#Resource">resources</a> allocated to service these inputs. This implies designs that have no contention points or central bottlenecks, resulting in the ability to shard or replicate components and distribute inputs among them. Reactive Systems support predictive, as well as Reactive, scaling algorithms by providing relevant live performance measures. They achieve <a href="http://www.reactivemanifesto.org/glossary#Elasticity">elasticity</a> in a cost-effective way on commodity hardware and software platforms.</p>

</section>

<section id="message-driven" class="trait">

    <p><strong>Message Driven: </strong>Reactive Systems rely on <a href="http://www.reactivemanifesto.org/glossary#Asynchronous">asynchronous</a> <a href="http://www.reactivemanifesto.org/glossary#Message-Driven">message-passing</a> to establish a boundary between components that ensures loose coupling, isolation, <a href="http://www.reactivemanifesto.org/glossary#Location-Transparency">location transparency</a>, and provides the means to delegate <a href="http://www.reactivemanifesto.org/glossary#Failure">errors</a> as messages. Employing explicit message-passing enables load management, elasticity, and flow control by shaping and monitoring the message queues in the system and applying <a href="http://www.reactivemanifesto.org/glossary#Back-Pressure">back-pressure</a> when necessary. Location transparent messaging as a means of communication makes it possible for the management of failure to work with the same constructs and semantics across a cluster or within a single host. <a href="http://www.reactivemanifesto.org/glossary#Non-Blocking">Non-blocking</a> communication allows recipients to only consume <a href="http://www.reactivemanifesto.org/glossary#Resource">resources</a> while active, leading to less system overhead.</p>
    
<img src="http://www.reactivemanifesto.org/images/reactive-traits.svg"/>

</section>

<section id="conclusion">

    <p>Large systems are composed of smaller ones and therefore depend on the Reactive properties of their constituents. This means that Reactive Systems apply design principles so these properties apply at all levels of scale, making them composable. The largest systems in the world rely upon architectures based on these properties and serve the needs of billions of people daily. It is time to apply these design principles consciously from the start instead of rediscovering them each time.</p>

</section>
</article>
