title: The Performance Inequality Gap, 2024
url: https://infrequently.org/2024/01/performance-inequality-gap-2024/
hash_url: 0676c7ccf1ab2b380641866789366d26
archive_date: 2024-01-31
og_image: https://infrequently.org/2024/01/performance-inequality-gap-2024/single_core_scores.png
description: How much HTML, CSS, and JavaScript can we afford? More than in years past, but much less than frontend developers are burdening users with.
favicon: https://infrequently.org/assets/images/favicons/ino-dark-opt-192x192.png
language: en_US

<p>It's time once again to update our priors regarding the global device and network situation. What's changed since last year? And how much more HTML, CSS, and (particularly) JavaScript can a new project afford?</p>
<p></p>
<h2 id="the-budget%2C-2024">The Budget, 2024 <a class="permalink" href="#the-budget%2C-2024">#</a></h2>
<p>In a departure from previous years, we'll evaluate two sets of baseline numbers for first-load under five seconds on 75<sup>th</sup> (<abbr>P75</abbr>) percentile devices and networks. First, we'll look at limits for JavaScript-heavy content, and separately we'll enunciate recommendations for markup-centric stacks.</p>
<p>This split decision was available via <a href="https://infrequently.org/2022/12/performance-baseline-2023/">last year's update</a>, but was somewhat buried. Going forward, I'll produce both as top-line guidance. The usual caveats also apply:</p>
<ul>
<li>Performance is a deep and nuanced domain, and much can go wrong beyond content size and composition.</li>
<li>How sites manage resources after-load can have a big impact on perceived performance.</li>
<li>Your audience may justify more stringent, or more relaxed, limits.</li>
</ul>
<p>With that stipulated, global baselines matter because many teams have low <a href="https://infrequently.org/2022/05/performance-management-maturity/">performance management maturity</a>, and today's popular frameworks – including some that market performance as a feature – <a href="https://infrequently.org/2023/02/the-market-for-lemons/">fail to ward against catastrophic results</a>.</p>
<p><em>Until and unless teams have better data about their performance, the global baseline budget should be enforced.</em></p>
<p>This isn't charity; it's how teams ensure products stay functional, accessible, and reliable in a market <a href="https://infrequently.org/2023/02/the-market-for-lemons/">awash in bullshit</a>. Limits help teams steer away from complexity and towards tools that generate simpler output that's easier to manage and repair.</p>
<h3 id="javascript-heavy">JavaScript-Heavy <a class="permalink" href="#javascript-heavy">#</a></h3>
<p>Since at least 2015, building JavaScript-first websites has been a predictably terrible idea, yet most of the sites I trace on a daily basis remain mired in script. For these sites, we have to factor in the heavy cost of running JavaScript on the client when describing how much content we can afford. HTML, CSS, images, and fonts can all be parsed and run at near wire speeds on low-end hardware, but JavaScript is at least three times more expensive, byte-for-byte.</p>
<p>Most sites, even those that aspire to be "lived in", feature short median sessions, which means we can't actually justify much in the way of up-front code, and first impressions always matter.</p>
<figure>
<picture class="preview">
  <source sizes="(max-width: 1200px) 70vw, 600px" srcset="https://infrequently.org/2023/02/the-market-for-lemons/depth-and-frequency-small.png?nf_resize=fit&amp;w=3600 2400w,
                  /2023/02/the-market-for-lemons/depth-and-frequency-small.png?nf_resize=fit&amp;w=2400 1600w,
                  /2023/02/the-market-for-lemons/depth-and-frequency-small.png?nf_resize=fit&amp;w=1800 1200w,
                  /2023/02/the-market-for-lemons/depth-and-frequency-small.png?nf_resize=fit&amp;w=1200   800w,
                  /2023/02/the-market-for-lemons/depth-and-frequency-small.png?nf_resize=fit&amp;w=900   600w,
                  /2023/02/the-market-for-lemons/depth-and-frequency-small.png?nf_resize=fit&amp;w=750   500w,
                  /2023/02/the-market-for-lemons/depth-and-frequency-small.png?nf_resize=fit&amp;w=600   400w">
<img src="https://infrequently.org/2023/02/the-market-for-lemons/depth-and-frequency-small.png" alt="Most sorts of sites have shallow sessions, making up-front script costs hard to justify." class="preview" decoding="async">
</source></picture>

  <figcaption>Most sorts of sites have shallow sessions, making up-front script costs hard to justify.</figcaption>
</figure>
<p>Over the estimated P75 global network, and targeting the slower of our two representative devices — and to hit five seconds to interactivity with only two critical-path network connections — we can afford ~1.3MiB of compressed content, comprised of:</p>
<ul>
<li>650KiB of HTML, CSS, images, and fonts</li>
<li>650KiB of JavaScript</li>
</ul>
<p>If we set the target to a much more reasonable three seconds, our total payload must fit in only ~730KiB, with no more than 365KiB of compressed JavaScript.</p>
<p>Similarly, if we keep the five second target but open five <abbr>TLS</abbr> connections, our budget would be closer to 1MiB. If the target were reset to three seconds with five connections, our total payload falls to ~460KiB, leaving only ~230KiB for scripts.</p>
<h3 id="markup-heavy">Markup-Heavy <a class="permalink" href="#markup-heavy">#</a></h3>
<p>Sites comprised mostly of markup (HTML and CSS) can afford a <em>lot</em> more, although CSS complexity and poorly-loaded fonts can still slow down otherwise quick content. Conservatively, to load in five seconds over, at most, two connections, we should try to keep content under 2.5MiB, including:</p>
<ul>
<li>2.4MiB of HTML, CSS, images, and fonts, and</li>
<li>100KiB of JavaScript.</li>
</ul>
<p>To hit a more reasonable three second first-load target with two connections, we should aim for a max 1.4MiB transfer, made up of:</p>
<ul>
<li>1.325MiB of HTML, CSS, etc., and</li>
<li>75KiB of JavaScript.</li>
</ul>
<p>These are generous targets. The blog you're reading <a href="https://www.webpagetest.org/video/compare.php?tests=240130_AiDcW6_5QC-r%3A1-c%3A0&amp;thumbSize=200&amp;ival=100&amp;end=full">loads over a single connection in ~1.2 seconds on the target device and network profile, consuming 120KiB of critical path resources to become interactive, only 8KiB of which is script</a>.</p>
<h3 id="calculate-your-own">Calculate Your Own <a class="permalink" href="#calculate-your-own">#</a></h3>
<p>As in years past, you can use <a href="https://infrequently.org/2024/01/performance-inequality-gap-2024/chart/index.html">the interactive estimate chart</a> to understand how connections and devices impact budgets. This year the chart has been updated to also allow you to select from JavaScript-heavy and JavaScript-light content composition, as well as updated network and device baselines (see below).</p>
<figure>
<a href="https://infrequently.org/2024/01/performance-inequality-gap-2024/chart/index.html" alt="&lt;em&gt;Tap to try the interactive version.&lt;/em&gt;" target="_new">
<picture class="preview">
  <source sizes="(max-width: 1200px) 70vw, 600px" srcset="https://infrequently.org/2024/01/performance-inequality-gap-2024/chart.png?nf_resize=fit&amp;w=3600 2400w,
                  https://infrequently.org/2024/01/performance-inequality-gap-2024/chart.png?nf_resize=fit&amp;w=2400 1600w,
                  https://infrequently.org/2024/01/performance-inequality-gap-2024/chart.png?nf_resize=fit&amp;w=1800 1200w,
                  https://infrequently.org/2024/01/performance-inequality-gap-2024/chart.png?nf_resize=fit&amp;w=1200   800w,
                  https://infrequently.org/2024/01/performance-inequality-gap-2024/chart.png?nf_resize=fit&amp;w=900   600w,
                  https://infrequently.org/2024/01/performance-inequality-gap-2024/chart.png?nf_resize=fit&amp;w=750   500w,
                  https://infrequently.org/2024/01/performance-inequality-gap-2024/chart.png?nf_resize=fit&amp;w=600   400w">
<img src="https://infrequently.org/2024/01/performance-inequality-gap-2024/chart.png" alt="&lt;em&gt;Tap to try the interactive version.&lt;/em&gt;" decoding="async" loading="lazy">
</source></picture>
</a>
  <figcaption><em>Tap to try the interactive version.</em></figcaption>
</figure>
<p>It's straightforward to understand the number of critical path network connections for a site from DevTools and to eyeball the content composition. Armed with that information, it's possible to use this estimator to quickly understand what sort of first-load experience users at the margins can expect. Give it a try!</p>
<h2 id="situation-report">Situation Report <a class="permalink" href="#situation-report">#</a></h2>
<p>These recommendations are not context-free, and you may disagree with them in whole or in part. To the extent that other estimates are more grounded, or based on different situational data, they may be more appropriate for specific products and teams. Many critiques are possible, both of the target (five seconds for first load), the sample population (worldwide internet users), and of the methodology (informed reckons). Regardless, I present the thinking behind them because it can provide teams with informed points of departure, and also because clarifying the ritual freakout taking place as <a href="https://web.dev/articles/inp"><abbr>INP</abbr> begins to put a price on JavaScript externalities.</a></p>
<p>It's clear that developers <a href="https://rviscomi.dev/2023/11/a-faster-web-in-2024/">are out of touch with market ground-truth</a>, but it's not obvious why. Understanding the differences in the experiences of wealthy developers versus working-class users helps to make the diffuse surface of the privilege bubble perceptible.</p>
<p>Engineering is the discipline of designing solutions under specific constraints. For the front end to improve, it must finally learn to operate within the envelope of what's possible on <em>most</em> devices.</p>
<h3 id="mobile">Mobile <a class="permalink" href="#mobile">#</a></h3>
<p>The "i" in iPhone stands for "inequality".</p>
<p>Owing to the chasm of global wealth inequality, premium devices are largely absent in markets with billions of users. India's iOS share has <a href="https://economictimes.indiatimes.com/tech/technology/apple-set-to-end-2023-with-7-market-share-for-iphones-in-android-dominated-india/articleshow/103532336.cms?from=mdr">surged to an all-time high of 7%</a> on the back of last-generation and refurbished devices. That's a market of 1.43 billion people where Apple <a href="https://www.counterpointresearch.com/insights/india-smartphone-share/">doesn't even crack the top five in terms of shipments</a>.</p>
<p>The Latin American (<abbr>LATAM</abbr>) region, home to more than 600 million people and <a href="https://www.statista.com/topics/7195/smartphone-market-in-latin-america/#topicOverview">nearly 200 million smartphones</a> shows a <a href="https://www.canalys.com/newsroom/latam-smartphone-market-q3-2023">similar market composition</a>:</p>
<figure>
<a href="https://www.counterpointresearch.com/research_portal/counterpoint-quarterly-smartphone-q4-2023/" alt="In &lt;abbr&gt;LATAM&lt;/abbr&gt;, iPhones make up less than 6% of total device shipments." target="_new">
<picture class="preview">
  <source sizes="(max-width: 1200px) 70vw, 600px" srcset="https://infrequently.org/2024/01/performance-inequality-gap-2024/latam_share_yoy_counterpoint.webp?nf_resize=fit&amp;w=3600 2400w,
                  https://infrequently.org/2024/01/performance-inequality-gap-2024/latam_share_yoy_counterpoint.webp?nf_resize=fit&amp;w=2400 1600w,
                  https://infrequently.org/2024/01/performance-inequality-gap-2024/latam_share_yoy_counterpoint.webp?nf_resize=fit&amp;w=1800 1200w,
                  https://infrequently.org/2024/01/performance-inequality-gap-2024/latam_share_yoy_counterpoint.webp?nf_resize=fit&amp;w=1200   800w,
                  https://infrequently.org/2024/01/performance-inequality-gap-2024/latam_share_yoy_counterpoint.webp?nf_resize=fit&amp;w=900   600w,
                  https://infrequently.org/2024/01/performance-inequality-gap-2024/latam_share_yoy_counterpoint.webp?nf_resize=fit&amp;w=750   500w,
                  https://infrequently.org/2024/01/performance-inequality-gap-2024/latam_share_yoy_counterpoint.webp?nf_resize=fit&amp;w=600   400w">
<img src="https://infrequently.org/2024/01/performance-inequality-gap-2024/latam_share_yoy_counterpoint.webp" alt="In &lt;abbr&gt;LATAM&lt;/abbr&gt;, iPhones make up less than 6% of total device shipments." decoding="async" loading="lazy">
</source></picture>
</a>
  <figcaption>In <abbr>LATAM</abbr>, iPhones make up less than 6% of total device shipments.</figcaption>
</figure>
<p>Everywhere wealth is unequally distributed, the haves <a href="https://www.statista.com/statistics/512863/smartphones-cell-phones-tablets-and-ereaders-brands-owned-by-affluent-americans/">read about it in Apple News over 5G while the have-nots struggle to get reliable 4G coverage for their Androids.</a> In <a href="https://assets.publishing.service.gov.uk/media/62a1cb0b8fa8f50395c0a0e7/Consumer_purchasing_behaviour_in_the_UK_smartphone_market_-_CMA_research_report_new.pdf">country after country (PDF)</a> the embedded inequality of our societies sorts ownership of devices by price, and brand through price segmentation.</p>
<p>This matters because the properties of those devices dominate the experiences we can deliver. In the U.S., the term "smartphone dependence" has been coined to describe folks without other ways to access the increasing fraction of essential services only available through the internet. Unsurprisingly, folks who can't afford other internet-connected devices or a fixed broadband subscription are also most likely to buy less expensive (and therefore slower) smartphones:</p>
<figure>
<a href="https://www.pewresearch.org/internet/fact-sheet/mobile/?tabId=tab-011fca0d-9756-4f48-b352-d58f343696bf" alt="undefined" target="_new">
<picture class="preview">
  <source sizes="(max-width: 1200px) 70vw, 600px" srcset="https://infrequently.org/2024/01/performance-inequality-gap-2024/us_smartphone_dependence_pew.webp?nf_resize=fit&amp;w=3600 2400w,
                  https://infrequently.org/2024/01/performance-inequality-gap-2024/us_smartphone_dependence_pew.webp?nf_resize=fit&amp;w=2400 1600w,
                  https://infrequently.org/2024/01/performance-inequality-gap-2024/us_smartphone_dependence_pew.webp?nf_resize=fit&amp;w=1800 1200w,
                  https://infrequently.org/2024/01/performance-inequality-gap-2024/us_smartphone_dependence_pew.webp?nf_resize=fit&amp;w=1200   800w,
                  https://infrequently.org/2024/01/performance-inequality-gap-2024/us_smartphone_dependence_pew.webp?nf_resize=fit&amp;w=900   600w,
                  https://infrequently.org/2024/01/performance-inequality-gap-2024/us_smartphone_dependence_pew.webp?nf_resize=fit&amp;w=750   500w,
                  https://infrequently.org/2024/01/performance-inequality-gap-2024/us_smartphone_dependence_pew.webp?nf_resize=fit&amp;w=600   400w">
<img src="https://infrequently.org/2024/01/performance-inequality-gap-2024/us_smartphone_dependence_pew.webp" alt="Missing alt text" decoding="async" loading="lazy">
</source></picture>
</a>
  <figcaption></figcaption>
</figure>
<p>As smartphone ownership and use grow, the front ends we deliver are ever-more mediated by the properties of those devices. The inequality between the high-end and low-end, even in wealthy countries, is only growing. What we choose to do in response defines what it means to practice <abbr>UX</abbr> engineering ethically.</p>
<h4 id="device-performance">Device Performance <a class="permalink" href="#device-performance">#</a></h4>
<p>Extending the <abbr title="system-on-chip">SoC</abbr> performance by price point series with another year's data, the picture remains ugly. The segments are roughly "fastest iPhone", "fastest Android", "budget", and "low-end":</p>
<figure>
<a href="https://infrequently.org/2024/01/performance-inequality-gap-2024/single_core_scores.png" alt="&lt;em&gt;Tap for a larger version.&lt;/em&gt;&lt;br&gt;Geekbench 5 single-core scores for each mobile price point." target="_new">
<picture class="preview">
  <source sizes="(max-width: 1200px) 70vw, 600px" srcset="https://infrequently.org/2024/01/performance-inequality-gap-2024/single_core_scores.png?nf_resize=fit&amp;w=3600 2400w,
                  https://infrequently.org/2024/01/performance-inequality-gap-2024/single_core_scores.png?nf_resize=fit&amp;w=2400 1600w,
                  https://infrequently.org/2024/01/performance-inequality-gap-2024/single_core_scores.png?nf_resize=fit&amp;w=1800 1200w,
                  https://infrequently.org/2024/01/performance-inequality-gap-2024/single_core_scores.png?nf_resize=fit&amp;w=1200   800w,
                  https://infrequently.org/2024/01/performance-inequality-gap-2024/single_core_scores.png?nf_resize=fit&amp;w=900   600w,
                  https://infrequently.org/2024/01/performance-inequality-gap-2024/single_core_scores.png?nf_resize=fit&amp;w=750   500w,
                  https://infrequently.org/2024/01/performance-inequality-gap-2024/single_core_scores.png?nf_resize=fit&amp;w=600   400w">
<img src="https://infrequently.org/2024/01/performance-inequality-gap-2024/single_core_scores.png" alt="&lt;em&gt;Tap for a larger version.&lt;/em&gt;&lt;br&gt;Geekbench 5 single-core scores for each mobile price point." decoding="async" loading="lazy">
</source></picture>
</a>
  <figcaption><em>Tap for a larger version.</em><br>Geekbench 5 single-core scores for each mobile price point.</figcaption>
</figure>
<p>Not only have fruity phones extended their single-core <abbr>CPU</abbr> performance lead over contemporary high-end Androids to <em>a four year advantage</em>, the performance-per-dollar curve remains unfavourable to Android buyers.</p>
<p>At the time of publication, the cheapest iPhone 15 Pro (the only device with the A17 Pro chip) is $999 <abbr>MSRP</abbr>, while the S23 (using the Snapdrago 8 gen 2) can be had for $860 from Samsung. This nets out to 2.32 points per dollar for the iPhone, but only 1.6 points per dollar for the S23.</p>
<p>Meanwhile a Samsung A24 that is $175 new, unlocked, and available on Amazon today scores a more reasonable 3.1 points per dollar on single-core performance, but is more than 4.25× slower than the leading contemporary iPhone.</p>
<p>The delta between the fastest iPhones and moderately price new devices rose from 1,522 points last year to 1,774 today.</p>
<p>Put another way, the performance gap between what devices the wealthy carry and what budget shoppers carry grew more this year (252 points) than the year-over-year gains from process and architecture at the volume price point (174 points). This is particularly depressing because single-core performance tends to determine the responsiveness of web app workloads.</p>
<p>A less pronounced version of the same story continues to play out in multi-core performance:</p>
<figure>
<a href="https://infrequently.org/2024/01/performance-inequality-gap-2024/multi_core_scores.png" alt="&lt;em&gt;Tap for a larger version.&lt;/em&gt;&lt;br&gt;Round and round we go: Android ecosystem &lt;abbr&gt;SoC&lt;/abbr&gt;s are improving, but the Performance Inequality Gap continues to grow. Even the fastest Androids are two-plus years behind iOS-ecosystem devices." target="_new">
<picture class="preview">
  <source sizes="(max-width: 1200px) 70vw, 600px" srcset="https://infrequently.org/2024/01/performance-inequality-gap-2024/multi_core_scores.png?nf_resize=fit&amp;w=3600 2400w,
                  https://infrequently.org/2024/01/performance-inequality-gap-2024/multi_core_scores.png?nf_resize=fit&amp;w=2400 1600w,
                  https://infrequently.org/2024/01/performance-inequality-gap-2024/multi_core_scores.png?nf_resize=fit&amp;w=1800 1200w,
                  https://infrequently.org/2024/01/performance-inequality-gap-2024/multi_core_scores.png?nf_resize=fit&amp;w=1200   800w,
                  https://infrequently.org/2024/01/performance-inequality-gap-2024/multi_core_scores.png?nf_resize=fit&amp;w=900   600w,
                  https://infrequently.org/2024/01/performance-inequality-gap-2024/multi_core_scores.png?nf_resize=fit&amp;w=750   500w,
                  https://infrequently.org/2024/01/performance-inequality-gap-2024/multi_core_scores.png?nf_resize=fit&amp;w=600   400w">
<img src="https://infrequently.org/2024/01/performance-inequality-gap-2024/multi_core_scores.png" alt="&lt;em&gt;Tap for a larger version.&lt;/em&gt;&lt;br&gt;Round and round we go: Android ecosystem &lt;abbr&gt;SoC&lt;/abbr&gt;s are improving, but the Performance Inequality Gap continues to grow. Even the fastest Androids are two-plus years behind iOS-ecosystem devices." decoding="async" loading="lazy">
</source></picture>
</a>
  <figcaption><em>Tap for a larger version.</em><br>Round and round we go: Android ecosystem <abbr>SoC</abbr>s are improving, but the Performance Inequality Gap continues to grow. Even the fastest Androids are two-plus years behind iOS-ecosystem devices.</figcaption>
</figure>
<p>Recent advantages in high-end Android multi-core performance have closed the gap to previous three-year gap to 18 months. Meanwhile, budget segment devices have finally started to see improvement (<a href="https://infrequently.org/2021/03/the-performance-inequality-gap/#:~:text=The%20good%20news%20is%20that%20this%20will%20change%20rapidly%20in%20the%20next%20few%20years.">as this series predicted</a>), thanks to hand-me-down architecture and process node improvements. That's where the good news ends.</p>
<p>The multi-core performance gap between i-devices and budget Androids grew considerably, with the score delta rising from 4,318 points last year to 4,936 points in 2023.</p>
<p>Looking forward, we can expect high-end Androids to at least stop falling further behind owing to <a href="https://www.androidauthority.com/snapdragon-8-gen-3-dimensity-9300-benchmarked-3395385/">a new focus on performance by Qualcomm's Snapdragon 8 gen 3 and MediaTek's Dimensity 9300 offerings</a>. This change is long, long overdue and will take years to filter down into positive outcomes for the rest of the ecosystem. Until that happens, the gap in experience for the wealthy versus the rest will not close.</p>
<p>iPhone owners live in a different world than high-end Android buyers, and light-years away what the bulk of the market experiences. No matter how you slice it, the performance inequality gap is growing for <abbr>CPU</abbr>-bound workloads like JavaScript-heavy web apps.</p>
<h4 id="networks">Networks <a class="permalink" href="#networks">#</a></h4>
<p>As ever, 2023 re-confirmed an essential truth when it comes to user experience: <a href="https://www.opensignal.com/2023/05/11/poor-connectivity-damages-the-mobile-app-business">when things are slow, users engage less often.</a> Doing a good job in an uneven network environment requires thinking about availability and engineering for resilience and a lightweight footprint — always better to avoid testing the radio gods than it is spending weeks or months appeasing them after the damage is done.</p>
<p>5G network deployment continues apace, but as with the arrival of 4G, it is happening unevenly and in ways and places that exacerbate (rather than lessen) performance inequality.</p>
<p>Data on mobile network evolution is sketchy, and the largest error bars in this series' analysis continue to reside in this section. Regardless, we can look industry summaries like the <a href="https://www.gsma.com/mobileeconomy/wp-content/uploads/2023/03/270223-The-Mobile-Economy-2023.pdf"><abbr>GSMA</abbr>'s report on "The Mobile Economy 2023" (PDF)</a> for a directional understanding that we can triangulate with other data points to develop a strong intuition.</p>
<p>For instance, <abbr>GSMA</abbr> predicts that 5G will only comprise half of connections by 2030. Meanwhile, McKinsey <a href="https://www.techtarget.com/whatis/feature/5-Predictions-about-5G-Adoption-in-2021-and-Beyond#:~:text=Regardless%20of%20the,the%205G%20revolution.%22">predicts</a> that high-quality 5G (networks that use 6GHz bands) will only cover a quarter of the world's population by 2030. Regulatory roadblocks are <a href="https://economictimes.indiatimes.com/industry/telecom/telecom-news/itu-reaches-agreement-to-open-new-6-ghz-spectrum-band-for-5g-6g/articleshow/106000126.cms">still being cleared</a>.</p>
<p>As we said in 2021, <em>"<a href="https://infrequently.org/2021/03/the-performance-inequality-gap/#oh-em-gee">4G is a miracle, 5G is a mirage</a>."</em></p>
<p>This doesn't mean that 4G is one thing, or that it's deployed evenly, or even that the <a href="https://www.opensignal.com/2023/06/29/more-usable-spectrum-boosts-the-4g-and-5g-experience">available spectrum will remain stable</a> within a single generation of radio technology. For example, India's network environment has continued to evolve since the <a href="https://www.kaiostech.com/reliance-jio-became-worlds-fastest-growing-mobile-network/">Reliance Jio revolution</a> that drove 4G into the mainstream and pushed the price of a mobile megabyte down by ~90% on <em>every</em> subcontinental carrier. But that's not the whole story! <a href="https://www.speedtest.net/global-index/india#mobile">Speedtest.net's data for India shows dramatic gains, for example</a>, and <a href="https://www.financialexpress.com/business/industry/mobile-download-speeds-india-moves-up-72-spots-in-global-ranking/3260813/">analysts credit this to improved infrastructure density, expanded spectrum, and back-haul improvements related to the 5G rollout</a> — all of which is to say that 4G users are getting better experiences than they did last year <em>because of</em> 5G's role in reducing contention.</p>
<figure>
<a href="https://www.speedtest.net/global-index/india#mobile" alt="India's speed test medians are moving quickly, but variance is orders-of-magnitude wide, with 5G penetration below 25% in the most populous areas." target="_new">
<picture class="preview">
  <source sizes="(max-width: 1200px) 70vw, 600px" srcset="https://infrequently.org/2024/01/performance-inequality-gap-2024/india_mobile_speedtest_data.webp?nf_resize=fit&amp;w=3600 2400w,
                  https://infrequently.org/2024/01/performance-inequality-gap-2024/india_mobile_speedtest_data.webp?nf_resize=fit&amp;w=2400 1600w,
                  https://infrequently.org/2024/01/performance-inequality-gap-2024/india_mobile_speedtest_data.webp?nf_resize=fit&amp;w=1800 1200w,
                  https://infrequently.org/2024/01/performance-inequality-gap-2024/india_mobile_speedtest_data.webp?nf_resize=fit&amp;w=1200   800w,
                  https://infrequently.org/2024/01/performance-inequality-gap-2024/india_mobile_speedtest_data.webp?nf_resize=fit&amp;w=900   600w,
                  https://infrequently.org/2024/01/performance-inequality-gap-2024/india_mobile_speedtest_data.webp?nf_resize=fit&amp;w=750   500w,
                  https://infrequently.org/2024/01/performance-inequality-gap-2024/india_mobile_speedtest_data.webp?nf_resize=fit&amp;w=600   400w">
<img src="https://infrequently.org/2024/01/performance-inequality-gap-2024/india_mobile_speedtest_data.webp" alt="India's speed test medians are moving quickly, but variance is orders-of-magnitude wide, with 5G penetration below 25% in the most populous areas." decoding="async" loading="lazy">
</source></picture>
</a>
  <figcaption>India's speed test medians are moving quickly, but variance is orders-of-magnitude wide, with 5G penetration below 25% in the most populous areas.</figcaption>
</figure>
<p>These sorts of gains are easy to miss if we look only at headline "4G vs. 5G" coverage, so it's important to level-set as new data becomes available. Improvements can arrive unevenly, with the "big" story happening slowly, long after the initial buzz of headlines wears off. These effects reward us for looking at P75+, not just means or medians, and intentionally updating priors on a regular basis.</p>
<p>Events can turn our intuitions on their heads, too. Japan is famously well connected. I've personally experienced rock-solid 4G through entire Tokyo subway journeys, <a href="https://en.wikipedia.org/wiki/Roppongi_Station">more than 40m underground</a> and with no hiccups. And yet, the network environment has been largely unchanged by the introduction of 5G. Having provisioned more than adequately in the 4G era, new technology isn't having the same impact from pent-up demand. But despite consistent performance, the quality of service for all users is distributed in a <em>much</em> more egalitarian way:</p>
<figure>
<a href="https://www.speedtest.net/global-index/japan#mobile" alt="Japan's network environment isn't the fastest, but is much more evenly distributed." target="_new">
<picture class="preview">
  <source sizes="(max-width: 1200px) 70vw, 600px" srcset="https://infrequently.org/2024/01/performance-inequality-gap-2024/japan_mobile_speedtest_data.webp?nf_resize=fit&amp;w=3600 2400w,
                  https://infrequently.org/2024/01/performance-inequality-gap-2024/japan_mobile_speedtest_data.webp?nf_resize=fit&amp;w=2400 1600w,
                  https://infrequently.org/2024/01/performance-inequality-gap-2024/japan_mobile_speedtest_data.webp?nf_resize=fit&amp;w=1800 1200w,
                  https://infrequently.org/2024/01/performance-inequality-gap-2024/japan_mobile_speedtest_data.webp?nf_resize=fit&amp;w=1200   800w,
                  https://infrequently.org/2024/01/performance-inequality-gap-2024/japan_mobile_speedtest_data.webp?nf_resize=fit&amp;w=900   600w,
                  https://infrequently.org/2024/01/performance-inequality-gap-2024/japan_mobile_speedtest_data.webp?nf_resize=fit&amp;w=750   500w,
                  https://infrequently.org/2024/01/performance-inequality-gap-2024/japan_mobile_speedtest_data.webp?nf_resize=fit&amp;w=600   400w">
<img src="https://infrequently.org/2024/01/performance-inequality-gap-2024/japan_mobile_speedtest_data.webp" alt="Japan's network environment isn't the fastest, but is much more evenly distributed." decoding="async" loading="lazy">
</source></picture>
</a>
  <figcaption>Japan's network environment isn't the fastest, but is much more evenly distributed.</figcaption>
</figure>
<p>Fleet device composition has big effects, owing to differences in signal-processing compute availability and spectrum compatibility. At a population level, these influences play out slowly as devices age out, but still have impressively positive impacts:</p>
<figure>
<a href="https://www.opensignal.com/2023/09/25/users-should-upgrade-their-iphone-to-have-the-best-mobile-network-experience" alt="Device impact on network performance is visible in Opensignal's iPhone dataset." target="_new">
<picture class="preview">
  <source sizes="(max-width: 1200px) 70vw, 600px" srcset="https://infrequently.org/2024/01/performance-inequality-gap-2024/opensignal_iphone_relative_network_speed.webp?nf_resize=fit&amp;w=3600 2400w,
                  https://infrequently.org/2024/01/performance-inequality-gap-2024/opensignal_iphone_relative_network_speed.webp?nf_resize=fit&amp;w=2400 1600w,
                  https://infrequently.org/2024/01/performance-inequality-gap-2024/opensignal_iphone_relative_network_speed.webp?nf_resize=fit&amp;w=1800 1200w,
                  https://infrequently.org/2024/01/performance-inequality-gap-2024/opensignal_iphone_relative_network_speed.webp?nf_resize=fit&amp;w=1200   800w,
                  https://infrequently.org/2024/01/performance-inequality-gap-2024/opensignal_iphone_relative_network_speed.webp?nf_resize=fit&amp;w=900   600w,
                  https://infrequently.org/2024/01/performance-inequality-gap-2024/opensignal_iphone_relative_network_speed.webp?nf_resize=fit&amp;w=750   500w,
                  https://infrequently.org/2024/01/performance-inequality-gap-2024/opensignal_iphone_relative_network_speed.webp?nf_resize=fit&amp;w=600   400w">
<img src="https://infrequently.org/2024/01/performance-inequality-gap-2024/opensignal_iphone_relative_network_speed.webp" alt="Device impact on network performance is visible in Opensignal's iPhone dataset." decoding="async" loading="lazy">
</source></picture>
</a>
  <figcaption>Device impact on network performance is visible in Opensignal's iPhone dataset.</figcaption>
</figure>
<p>As inequality grows, <a href="https://www.financialexpress.com/business/industry/mobile-download-speeds-india-moves-up-72-spots-in-global-ranking/3260813/#:~:text=In%20fact%2C%20in,of%20385.50%20Mbps.">averages and "generation" tags can become illusory and misleading</a>. Our own experiences are no guide; we've got to keep our hands in the data to understand the texture of the world.</p>
<p>So, with all of that as prelude, what <em>can</em> we say about where the mobile network baseline should be set? In a departure from years prior, I'm going to use a unified network estimate (see below). You'll have to read on for what it is! But it won't be based on the sort of numbers that folks explicitly running speed tests see; those aren't real life.</p>

<h4 id="market-factors">Market Factors <a class="permalink" href="#market-factors">#</a></h4>
<p>The market forces this series <a href="https://infrequently.org/2017/10/can-you-afford-it-real-world-web-performance-budgets/#global-ground-truth">previewed in 2017</a> have played out in roughly a straight line: smartphone penetration in emerging markets is approaching saturation, ensuring a growing fraction of purchases are made by upgrade shoppers. Those who upgrade see more value in their phones and save to buy better second and third devices. Combined with the <a href="https://en.wikipedia.org/wiki/IPhone_X#:~:text=At%20the%20time%20of%20its,local%20sales%20and%20import%20taxes.">emergence</a> and <a href="https://www.counterpointresearch.com/insights/premium-smartphone-asp-reaches-record-q2-high/">growth of the "ultra premium" segment</a>, average selling prices (<abbr>ASP</abbr>s) have risen.</p>
<p>2022 and 2023 have established an inflection point in the regard, with worldwide average selling prices <a href="https://www.idc.com/getdoc.jsp?containerId=prUS51430223">jumping to more than $430</a>, up from $300-$350 for much of the decade prior. Some price appreciation has been <a href="https://www.technavio.com/report/smartphone-market-industry-analysis">due to transient impacts of the U.S./China trade wars</a>, but most of it appears driven by iOS <abbr>ASP</abbr>s which peaked above $1,000 for the first time in 2023. Android <abbr>ASP</abbr>s, meanwhile, continued a gradual rise to nearly $300, up from $250 five years ago.</p>
<figure>
<a href="https://www.idc.com/getdoc.jsp?containerId=prUS51430223" alt="undefined" target="_new">
<picture class="preview">
  <source sizes="(max-width: 1200px) 70vw, 600px" srcset="https://infrequently.org/2024/01/performance-inequality-gap-2024/idc_forecast.png?nf_resize=fit&amp;w=3600 2400w,
                  https://infrequently.org/2024/01/performance-inequality-gap-2024/idc_forecast.png?nf_resize=fit&amp;w=2400 1600w,
                  https://infrequently.org/2024/01/performance-inequality-gap-2024/idc_forecast.png?nf_resize=fit&amp;w=1800 1200w,
                  https://infrequently.org/2024/01/performance-inequality-gap-2024/idc_forecast.png?nf_resize=fit&amp;w=1200   800w,
                  https://infrequently.org/2024/01/performance-inequality-gap-2024/idc_forecast.png?nf_resize=fit&amp;w=900   600w,
                  https://infrequently.org/2024/01/performance-inequality-gap-2024/idc_forecast.png?nf_resize=fit&amp;w=750   500w,
                  https://infrequently.org/2024/01/performance-inequality-gap-2024/idc_forecast.png?nf_resize=fit&amp;w=600   400w">
<img src="https://infrequently.org/2024/01/performance-inequality-gap-2024/idc_forecast.png" alt="Missing alt text" decoding="async" loading="lazy">
</source></picture>
</a>
  <figcaption></figcaption>
</figure>
<p>A <a href="https://www.counterpointresearch.com/insights/global-smartphone-market-reaches-its-lowest-q3-levels-in-a-decade-apples-share-at-16/">weak market for handsets in 2023</a>, plus stable sales for iOS, had an notable impact on prices. <abbr>IDC</abbr> expects global average prices to fall back below $400 by 2027 as Android volumes increase from an unusually soft 2023.</p>
<figure>
<a href="https://www.counterpointresearch.com/research_portal/counterpoint-quarterly-smartphone-q4-2023/" alt="Counterpoint data shows declining sales in both 2022 and 2023." target="_new">
<picture class="preview">
  <source sizes="(max-width: 1200px) 70vw, 600px" srcset="https://infrequently.org/2024/01/performance-inequality-gap-2024/smartphone_shipments_2023.webp?nf_resize=fit&amp;w=3600 2400w,
                  https://infrequently.org/2024/01/performance-inequality-gap-2024/smartphone_shipments_2023.webp?nf_resize=fit&amp;w=2400 1600w,
                  https://infrequently.org/2024/01/performance-inequality-gap-2024/smartphone_shipments_2023.webp?nf_resize=fit&amp;w=1800 1200w,
                  https://infrequently.org/2024/01/performance-inequality-gap-2024/smartphone_shipments_2023.webp?nf_resize=fit&amp;w=1200   800w,
                  https://infrequently.org/2024/01/performance-inequality-gap-2024/smartphone_shipments_2023.webp?nf_resize=fit&amp;w=900   600w,
                  https://infrequently.org/2024/01/performance-inequality-gap-2024/smartphone_shipments_2023.webp?nf_resize=fit&amp;w=750   500w,
                  https://infrequently.org/2024/01/performance-inequality-gap-2024/smartphone_shipments_2023.webp?nf_resize=fit&amp;w=600   400w">
<img src="https://infrequently.org/2024/01/performance-inequality-gap-2024/smartphone_shipments_2023.webp" alt="Counterpoint data shows declining sales in both 2022 and 2023." decoding="async" loading="lazy">
</source></picture>
</a>
  <figcaption>Counterpoint data shows declining sales in both 2022 and 2023.</figcaption>
</figure>
<figure>
<a href="https://www.counterpointresearch.com/research_portal/counterpoint-quarterly-smartphone-q4-2023/" alt="Shipment growth in late 2023 and beyond is coming from emerging markets like the Middle East and Africa. Samsung's A-series mid-tier is doing particularly well." target="_new">
<picture class="preview">
  <source sizes="(max-width: 1200px) 70vw, 600px" srcset="https://infrequently.org/2024/01/performance-inequality-gap-2024/return_to_growth_driven_by_em.webp?nf_resize=fit&amp;w=3600 2400w,
                  https://infrequently.org/2024/01/performance-inequality-gap-2024/return_to_growth_driven_by_em.webp?nf_resize=fit&amp;w=2400 1600w,
                  https://infrequently.org/2024/01/performance-inequality-gap-2024/return_to_growth_driven_by_em.webp?nf_resize=fit&amp;w=1800 1200w,
                  https://infrequently.org/2024/01/performance-inequality-gap-2024/return_to_growth_driven_by_em.webp?nf_resize=fit&amp;w=1200   800w,
                  https://infrequently.org/2024/01/performance-inequality-gap-2024/return_to_growth_driven_by_em.webp?nf_resize=fit&amp;w=900   600w,
                  https://infrequently.org/2024/01/performance-inequality-gap-2024/return_to_growth_driven_by_em.webp?nf_resize=fit&amp;w=750   500w,
                  https://infrequently.org/2024/01/performance-inequality-gap-2024/return_to_growth_driven_by_em.webp?nf_resize=fit&amp;w=600   400w">
<img src="https://infrequently.org/2024/01/performance-inequality-gap-2024/return_to_growth_driven_by_em.webp" alt="Shipment growth in late 2023 and beyond is coming from emerging markets like the Middle East and Africa. Samsung's A-series mid-tier is doing particularly well." decoding="async" loading="lazy">
</source></picture>
</a>
  <figcaption>Shipment growth in late 2023 and beyond is coming from emerging markets like the Middle East and Africa. Samsung's A-series mid-tier is doing particularly well.</figcaption>
</figure>
<p>Despite falling sales, distribution of Android versus iOS sales remains largely unchanged:</p>
<figure>
<a href="https://www.counterpointresearch.com/insights/global-smartphone-os-market-share/" alt="Android sales reliably constitute 80-85% of worldwide volume." target="_new">
<picture class="preview">
  <source sizes="(max-width: 1200px) 70vw, 600px" srcset="https://infrequently.org/2024/01/performance-inequality-gap-2024/counterpoint_smartphone_sales_by_OS_Q3-2023.webp?nf_resize=fit&amp;w=3600 2400w,
                  https://infrequently.org/2024/01/performance-inequality-gap-2024/counterpoint_smartphone_sales_by_OS_Q3-2023.webp?nf_resize=fit&amp;w=2400 1600w,
                  https://infrequently.org/2024/01/performance-inequality-gap-2024/counterpoint_smartphone_sales_by_OS_Q3-2023.webp?nf_resize=fit&amp;w=1800 1200w,
                  https://infrequently.org/2024/01/performance-inequality-gap-2024/counterpoint_smartphone_sales_by_OS_Q3-2023.webp?nf_resize=fit&amp;w=1200   800w,
                  https://infrequently.org/2024/01/performance-inequality-gap-2024/counterpoint_smartphone_sales_by_OS_Q3-2023.webp?nf_resize=fit&amp;w=900   600w,
                  https://infrequently.org/2024/01/performance-inequality-gap-2024/counterpoint_smartphone_sales_by_OS_Q3-2023.webp?nf_resize=fit&amp;w=750   500w,
                  https://infrequently.org/2024/01/performance-inequality-gap-2024/counterpoint_smartphone_sales_by_OS_Q3-2023.webp?nf_resize=fit&amp;w=600   400w">
<img src="https://infrequently.org/2024/01/performance-inequality-gap-2024/counterpoint_smartphone_sales_by_OS_Q3-2023.webp" alt="Android sales reliably constitute 80-85% of worldwide volume." decoding="async" loading="lazy">
</source></picture>
</a>
  <figcaption>Android sales reliably constitute 80-85% of worldwide volume.</figcaption>
</figure>
<figure>
<a href="https://www.statista.com/statistics/245191/market-share-of-mobile-operating-systems-for-smartphone-sales-in-australia/" alt="Even in rich nations like Australia and the &lt;a href='https://www.statista.com/statistics/262179/market-share-held-by-mobile-operating-systems-in-the-united-kingdom/'&gt;the U.K.&lt;/a&gt;, iPhones account for less than half of sales. Predictably, they are over-represented in analytics and logs owing to wealth-related factors including superior network access and performance hysteresis." target="_new">
<picture class="preview">
  <source sizes="(max-width: 1200px) 70vw, 600px" srcset="https://infrequently.org/2024/01/performance-inequality-gap-2024/au_smartphone_share_by_os.webp?nf_resize=fit&amp;w=3600 2400w,
                  https://infrequently.org/2024/01/performance-inequality-gap-2024/au_smartphone_share_by_os.webp?nf_resize=fit&amp;w=2400 1600w,
                  https://infrequently.org/2024/01/performance-inequality-gap-2024/au_smartphone_share_by_os.webp?nf_resize=fit&amp;w=1800 1200w,
                  https://infrequently.org/2024/01/performance-inequality-gap-2024/au_smartphone_share_by_os.webp?nf_resize=fit&amp;w=1200   800w,
                  https://infrequently.org/2024/01/performance-inequality-gap-2024/au_smartphone_share_by_os.webp?nf_resize=fit&amp;w=900   600w,
                  https://infrequently.org/2024/01/performance-inequality-gap-2024/au_smartphone_share_by_os.webp?nf_resize=fit&amp;w=750   500w,
                  https://infrequently.org/2024/01/performance-inequality-gap-2024/au_smartphone_share_by_os.webp?nf_resize=fit&amp;w=600   400w">
<img src="https://infrequently.org/2024/01/performance-inequality-gap-2024/au_smartphone_share_by_os.webp" alt="Even in rich nations like Australia and the &lt;a href='https://www.statista.com/statistics/262179/market-share-held-by-mobile-operating-systems-in-the-united-kingdom/'&gt;the U.K.&lt;/a&gt;, iPhones account for less than half of sales. Predictably, they are over-represented in analytics and logs owing to wealth-related factors including superior network access and performance hysteresis." decoding="async" loading="lazy">
</source></picture>
</a>
  <figcaption>Even in rich nations like Australia and the <a href="https://www.statista.com/statistics/262179/market-share-held-by-mobile-operating-systems-in-the-united-kingdom/">the U.K.</a>, iPhones account for less than half of sales. Predictably, they are over-represented in analytics and logs owing to wealth-related factors including superior network access and performance hysteresis.</figcaption>
</figure>
<p>Smartphone replacement rates have remained roughly in line with previous years, although we should expect elongation in the data from next year. <a href="https://www.sellcell.com/blog/how-often-do-people-upgrade-their-phone-2023-statistics/">Survey reports</a> and market analysts continue to estimate average replacement at 3-4 years, depending on segment. Premium devices last longer, and a higher fraction of devices may be older in wealthy geographies. Combined with <a href="https://www2.deloitte.com/us/en/insights/economy/consumer-pulse/state-of-the-us-consumer.html">discretionary spending pressure</a> and <a href="https://www.ons.gov.uk/economy/inflationandpriceindices/articles/costofliving/latestinsights">inflationary impacts on household budgets</a>, consumer intent to spend on electronics has taken a hit, which will be felt in device lifetime extension until conditions improve. <a href="https://www.counterpointresearch.com/insights/apple-refurbished-smartphone-volumes-grew-16-yoy-globally-in-2022/">Increasing demand for refurbished devices</a> also adds to observable device aging.</p>
<p>The data paints a substantially similar picture to previous years: the web is experienced on devices that are slower and older than those carried by affluent developers and corporate directors whose purchasing decisions are not impacted by transitory inflation.</p>
<p>To serve users effectively, we must do extra work to <a href="https://glazkov.com/2023/07/30/live-as-our-customer/">live as our customers do</a>.</p>
<h4 id="test-device-recommendations">Test Device Recommendations <a class="permalink" href="#test-device-recommendations">#</a></h4>
<p>Re-using <a href="https://infrequently.org/2022/12/performance-baseline-2023/#devices-1">last year's P75 device calculus</a>, our estimate is based on a device sold new, unlocked for the mid-2020 to mid-2021 global <abbr>ASP</abbr> of ~$350-375.</p>
<p>Representative examples from that time period include the <a href="https://www.gsmarena.com/samsung_galaxy_a51-9963.php">Samsung Galaxy A51</a> and the <a href="https://www.gsmarena.com/google_pixel_4a-10123.php">Pixel 4a</a>. Neither model featured 5G, and we cannot expect 5G to play a significant role in worldwide baselines for at least the next several years.</p>
<p>The A51 featured <a href="https://www.gsmarena.com/samsung_galaxy_a51-9963.php#:~:text=Octa%2Dcore%20(4x2.3%20GHz%20Cortex%2DA73%20%26%204x1.7%20GHz%20Cortex%2DA53)">eight slow cores (4x2.3 GHz Cortex-A73 and 4x1.7 GHz Cortex-A53) on a 10nm process</a>:</p>
<figure>
<a href="https://browser.geekbench.com/v6/cpu/compare/350184?baseline=3639070" alt="Geekbench 6 scores for the Galaxy A51 versus today's leading device." target="_new">
<picture class="preview">
  <source sizes="(max-width: 1200px) 70vw, 600px" srcset="https://infrequently.org/2024/01/performance-inequality-gap-2024/a51_vs_iphone_15_pro.webp?nf_resize=fit&amp;w=3600 2400w,
                  https://infrequently.org/2024/01/performance-inequality-gap-2024/a51_vs_iphone_15_pro.webp?nf_resize=fit&amp;w=2400 1600w,
                  https://infrequently.org/2024/01/performance-inequality-gap-2024/a51_vs_iphone_15_pro.webp?nf_resize=fit&amp;w=1800 1200w,
                  https://infrequently.org/2024/01/performance-inequality-gap-2024/a51_vs_iphone_15_pro.webp?nf_resize=fit&amp;w=1200   800w,
                  https://infrequently.org/2024/01/performance-inequality-gap-2024/a51_vs_iphone_15_pro.webp?nf_resize=fit&amp;w=900   600w,
                  https://infrequently.org/2024/01/performance-inequality-gap-2024/a51_vs_iphone_15_pro.webp?nf_resize=fit&amp;w=750   500w,
                  https://infrequently.org/2024/01/performance-inequality-gap-2024/a51_vs_iphone_15_pro.webp?nf_resize=fit&amp;w=600   400w">
<img src="https://infrequently.org/2024/01/performance-inequality-gap-2024/a51_vs_iphone_15_pro.webp" alt="Geekbench 6 scores for the Galaxy A51 versus today's leading device." decoding="async" loading="lazy">
</source></picture>
</a>
  <figcaption>Geekbench 6 scores for the Galaxy A51 versus today's leading device.</figcaption>
</figure>
<p><a href="https://www.gsmarena.com/google_pixel_4a-10123.php#:~:text=Octa%2Dcore%20(2x2.2%20GHz%20Kryo%20470%20Gold%20%26%206x1.8%20GHz%20Kryo%20470%20Silver)">The Pixel 4a's slow, eight-core big.LITTLE configuration was fabricated on an 8nm process</a>:</p>
<figure>
<a href="https://browser.geekbench.com/v6/cpu/compare/4295850?baseline=3639070" alt="Google spent more on the &lt;abbr&gt;SoC&lt;/abbr&gt; for the Pixel 4a and enjoyed a later launch date, boosting performance relative to the A51." target="_new">
<picture class="preview">
  <source sizes="(max-width: 1200px) 70vw, 600px" srcset="https://infrequently.org/2024/01/performance-inequality-gap-2024/pixel_4a_vs_iphone_15_pro.webp?nf_resize=fit&amp;w=3600 2400w,
                  https://infrequently.org/2024/01/performance-inequality-gap-2024/pixel_4a_vs_iphone_15_pro.webp?nf_resize=fit&amp;w=2400 1600w,
                  https://infrequently.org/2024/01/performance-inequality-gap-2024/pixel_4a_vs_iphone_15_pro.webp?nf_resize=fit&amp;w=1800 1200w,
                  https://infrequently.org/2024/01/performance-inequality-gap-2024/pixel_4a_vs_iphone_15_pro.webp?nf_resize=fit&amp;w=1200   800w,
                  https://infrequently.org/2024/01/performance-inequality-gap-2024/pixel_4a_vs_iphone_15_pro.webp?nf_resize=fit&amp;w=900   600w,
                  https://infrequently.org/2024/01/performance-inequality-gap-2024/pixel_4a_vs_iphone_15_pro.webp?nf_resize=fit&amp;w=750   500w,
                  https://infrequently.org/2024/01/performance-inequality-gap-2024/pixel_4a_vs_iphone_15_pro.webp?nf_resize=fit&amp;w=600   400w">
<img src="https://infrequently.org/2024/01/performance-inequality-gap-2024/pixel_4a_vs_iphone_15_pro.webp" alt="Google spent more on the &lt;abbr&gt;SoC&lt;/abbr&gt; for the Pixel 4a and enjoyed a later launch date, boosting performance relative to the A51." decoding="async" loading="lazy">
</source></picture>
</a>
  <figcaption>Google spent more on the <abbr>SoC</abbr> for the Pixel 4a and enjoyed a later launch date, boosting performance relative to the A51.</figcaption>
</figure>
<p><a href="https://www.androidpolice.com/why-google-pixel-phones-hardware-do-not-sell/">Pixels have never sold well,</a> and Google's focus on strong <qabbr>SoC performance per dollar was sadly not replicated across the Android ecosystem, forcing us to use the A51 as our stand-in.</qabbr></p>
<p>Devices within the envelope of our attention are 15-25% as fast as those carried by programmers and their bosses — even in wealthy markets.</p>
<p>The Galaxy may be <a href="https://browser.geekbench.com/v6/cpu/compare/4301594?baseline=442665">slightly faster</a> than last year's <a href="https://infrequently.org/2022/12/performance-baseline-2023/#:~:text=The%20best%20analogue%20you%20can%20buy%20for%20a%20representative%20P75%20device%20today%20are%20~%24200%20Androids%20from%20the%20last%20year%20or%20two%2C%20such%20as%20the%20Samsung%20Galaxy%20A50%20and%20the%20Nokia%20G11.">recommendation</a> of the <a href="https://www.gsmarena.com/samsung_galaxy_a50-9554.php">Galaxy A50 for testing</a>, but the picture is muddy:</p>
<figure>
<a href="https://browser.geekbench.com/v5/cpu/compare/22080983?baseline=22095605" alt="Geekbench 5 shows almost no improvement between the A50 and the A51." target="_new">
<picture class="preview">
  <source sizes="(max-width: 1200px) 70vw, 600px" srcset="https://infrequently.org/2024/01/performance-inequality-gap-2024/a50_vs_a51_gb5.webp?nf_resize=fit&amp;w=3600 2400w,
                  https://infrequently.org/2024/01/performance-inequality-gap-2024/a50_vs_a51_gb5.webp?nf_resize=fit&amp;w=2400 1600w,
                  https://infrequently.org/2024/01/performance-inequality-gap-2024/a50_vs_a51_gb5.webp?nf_resize=fit&amp;w=1800 1200w,
                  https://infrequently.org/2024/01/performance-inequality-gap-2024/a50_vs_a51_gb5.webp?nf_resize=fit&amp;w=1200   800w,
                  https://infrequently.org/2024/01/performance-inequality-gap-2024/a50_vs_a51_gb5.webp?nf_resize=fit&amp;w=900   600w,
                  https://infrequently.org/2024/01/performance-inequality-gap-2024/a50_vs_a51_gb5.webp?nf_resize=fit&amp;w=750   500w,
                  https://infrequently.org/2024/01/performance-inequality-gap-2024/a50_vs_a51_gb5.webp?nf_resize=fit&amp;w=600   400w">
<img src="https://infrequently.org/2024/01/performance-inequality-gap-2024/a50_vs_a51_gb5.webp" alt="Geekbench 5 shows almost no improvement between the A50 and the A51." decoding="async" loading="lazy">
</source></picture>
</a>
  <figcaption>Geekbench 5 shows almost no improvement between the A50 and the A51.</figcaption>
</figure>
<figure>
<a href="https://browser.geekbench.com/v6/cpu/compare/4302358?baseline=4260956" alt="Geekbench 6 shows the same story within the margin of error. The low-end is stagnant, and still &lt;a href='https://www.statista.com/statistics/934471/smartphone-shipments-by-price-category-worldwide/' target='_new'&gt;30% of worldwide volume&lt;/a&gt;." target="_new">
<picture class="preview">
  <source sizes="(max-width: 1200px) 70vw, 600px" srcset="https://infrequently.org/2024/01/performance-inequality-gap-2024/a50_vs_a51_gb6.webp?nf_resize=fit&amp;w=3600 2400w,
                  https://infrequently.org/2024/01/performance-inequality-gap-2024/a50_vs_a51_gb6.webp?nf_resize=fit&amp;w=2400 1600w,
                  https://infrequently.org/2024/01/performance-inequality-gap-2024/a50_vs_a51_gb6.webp?nf_resize=fit&amp;w=1800 1200w,
                  https://infrequently.org/2024/01/performance-inequality-gap-2024/a50_vs_a51_gb6.webp?nf_resize=fit&amp;w=1200   800w,
                  https://infrequently.org/2024/01/performance-inequality-gap-2024/a50_vs_a51_gb6.webp?nf_resize=fit&amp;w=900   600w,
                  https://infrequently.org/2024/01/performance-inequality-gap-2024/a50_vs_a51_gb6.webp?nf_resize=fit&amp;w=750   500w,
                  https://infrequently.org/2024/01/performance-inequality-gap-2024/a50_vs_a51_gb6.webp?nf_resize=fit&amp;w=600   400w">
<img src="https://infrequently.org/2024/01/performance-inequality-gap-2024/a50_vs_a51_gb6.webp" alt="Geekbench 6 shows the same story within the margin of error. The low-end is stagnant, and still &lt;a href='https://www.statista.com/statistics/934471/smartphone-shipments-by-price-category-worldwide/' target='_new'&gt;30% of worldwide volume&lt;/a&gt;." decoding="async" loading="lazy">
</source></picture>
</a>
  <figcaption>Geekbench 6 shows the same story within the margin of error. The low-end is stagnant, and still <a href="https://www.statista.com/statistics/934471/smartphone-shipments-by-price-category-worldwide/" target="_new">30% of worldwide volume</a>.</figcaption>
</figure>
<p>If you're building a test lab today, refurbished A51s can be had for ~$150. Even better, the newer <a href="https://browser.geekbench.com/v6/cpu/compare/3826666?baseline=350184">Nokia G100</a> can be had for as little as $100, and it's <a href="https://www.nokia.com/phones/en_us/nokia-g-100?sku=F22CF51022200">faithful</a> to the sluggish original in <a href="https://www.gsmarena.com/compare.php3?idPhone1=9963&amp;idPhone2=12373">nearly every respect</a>.</p>
<p>If your test bench is based on last year's recommended A50 or <a href="https://www.gsmarena.com/nokia_g11-11358.php">Nokia G11</a>, I do not recommend upgrading in 2024. The absolute gains are so slight that the difference will be hard to feel, and bench stability has a value all its own. Looking forward, we can also predict that our bench performance will be stable until 2025.</p>
<p>Claims about how "performant" modern front-end tools are have to be evaluated in this slow, stagnant context.</p>
<h3 id="desktop">Desktop <a class="permalink" href="#desktop">#</a></h3>
<p>It's a bit easier to understand the Desktop situation because the Edge telemetry I have access to provides statistically significant insight into <a href="https://www.statista.com/statistics/576473/united-states-quarterly-pc-shipment-share-apple/">85+% of the market</a>.</p>
<h4 id="device-performance-1">Device Performance <a class="permalink" href="#device-performance-1">#</a></h4>
<p>The <abbr>TL;DR</abbr> for desktop performance is that Edge telemetry puts ~45% of devices in a "low-end" bucket, meaning they have &lt;= 4 cores or &lt;= 4GB of RAM.</p>
<table class="summary">
  <thead>
    <tr>
      <td>Device Tier</td>
      <td>Fleet %</td>
      <td>Definition</td>
    </tr>
  </thead>
  <tbody>
    <tr>
      <td>Low-end</td>
      <td>45%</td>
      <td>Either:<br>&lt;= 4 cores, or<br>&lt;= 4GB RAM</td>
    </tr>
    <tr>
      <td>Medium</td>
      <td>48%</td>
      <td><abbr>HDD</abbr> (not <abbr>SSD</abbr>), or<br>4-16 GB RAM, or<br>4-8 cores</td>
    </tr>
    <tr>
      <td>High</td>
      <td>7%</td>
      <td><abbr>SSD</abbr> +<br>&gt; 8 cores +<br>&gt; 16GB RAM</td>
    </tr>
  </tbody>
</table>
<p>20% of users are on <abbr>HDD</abbr>s (not <abbr>SSD</abbr>s) and nearly all of those users also have low (and slow) cores.</p>
<p>You might be tempted to dismiss this data because it doesn't include Macs, which are faster than the PC cohort. Recall, however, that the snapshot also excludes ChromeOS.</p>
<p>ChromeOS share has veered wildly in recent years, representing 50%-200% of Mac shipments in a given per quarter. In '21 and '22, ChromeOS shipments regularly doubled Mac sales. Despite post-pandemic mean reversion, <a href="https://www.idc.com/getdoc.jsp?containerId=IDC_P36344">according to <abbr>IDC</abbr></a> ChromeOS devices outsold Macs ~5.7M to ~4.7M in 2023 Q2. The trend reversed in Q3, with Macs almost doubling ChromeOS sales, but slow ChromeOS devices aren't going away and, from a population perspective, more than offset Macs today. Analysts also <a href="https://www.idc.com/promo/pcdforecast">predict growth in the low end of the market as educational institutions begin to refresh their past purchases.</a></p>
<h4 id="networks-1">Networks <a class="permalink" href="#networks-1">#</a></h4>
<p>Desktop-attached networks <a href="https://www.fiercetelecom.com/broadband/ookla-global-fixed-download-speeds-nearly-doubled-2022">continue to improve</a>, notably <a href="https://www.allconnect.com/blog/broadband-availability-by-type">in the U.S.</a> Regulatory intervention and subsidies have done much to spur enhancements in access to U.S. fixed broadband, although <a href="https://www.allconnect.com/blog/broadband-availability-by-type#:~:text=Top%20and%20bottom%20states%20for%20speed">disparities in access remain</a> and the gains <a href="https://www.cbsnews.com/news/affordable-internet-service-could-be-lost-fcc-program-to-run-out-of-funds/">may not persist</a>.</p>
<p>This suggests that it's time to also bump our baseline for desktop tests beyond the 5Mbps/1Mbps/28ms configuration that <a href="https://www.webpagetest.org/">WebPageTest.org's "Cable" profile</a> has defaulted to for desktop tests.</p>
<p>How far should we bump it? Publicly available data is unclear, and I've come to find out that Edge's telemetry lacks good network observation statistics (doh!); Windows telemetry doesn't capture a proxy for network quality, I no longer have access to Chrome's data, the <a href="https://developer.chrome.com/docs/crux/api#effective_connection_type">population-level telemetry available from CrUX is unhelpful</a>, and <a href="https://www.fcc.gov/reports-research/reports/measuring-broadband-america/measuring-fixed-broadband-twelfth-report#:~:text=Chart%2016.2%3A%20The%20ratio%20of%2070/70%20consistent%20download%20speed%20to%20advertised%20download%20speed.">telcos li</a>...er...sorry, <em>"market their products in accordance with local laws and advertising standards."</em> All of this makes it difficult to construct an estimate.</p>
<p>One option is to use a population-level assessment of medians from <a href="https://www.speedtest.net/global-index">something like the Speedtest.net data</a> and then construct a histogram from median speeds. This is both time-consuming and error-prone, as population-level data varies widely across the world. Emerging markets with high mobile internet use and dense populations <a href="https://www.fiercewireless.com/wireless/indias-top-2-mobile-carriers-fight-supremacy-fixed-broadband">can feature</a> poor fixed-line broadband penetration <a href="https://www.opensignal.com/2023/11/22/closing-the-gap-fixed-broadbands-role-in-global-progress">compared with Western markets</a>.</p>
<p>Another option is to mathematically hand-wave using the best evidence we can get. This might allow us to reconstruct probable P75 and P90 values if we know something about the historical distribution of connections. From there, we can gut-check using other spot data. To do this, we need to assume some data set is representative, a fraught decision all its own. Biting the bullet, we could start from the Speedtest.net global survey data, which currently fails to provide anything but medians (P50):</p>
<figure>
<a href="https://www.speedtest.net/global-index" alt="Speedtest.net's global median values are unhelpful on their own, both because they represent users who are testing for speed (and not organic throughput) and because they don't give us a fuller understanding of the distribution." target="_new">
<picture class="preview">
  <source sizes="(max-width: 1200px) 70vw, 600px" srcset="https://infrequently.org/2024/01/performance-inequality-gap-2024/global_fixed_speedtest_medians.webp?nf_resize=fit&amp;w=3600 2400w,
                  https://infrequently.org/2024/01/performance-inequality-gap-2024/global_fixed_speedtest_medians.webp?nf_resize=fit&amp;w=2400 1600w,
                  https://infrequently.org/2024/01/performance-inequality-gap-2024/global_fixed_speedtest_medians.webp?nf_resize=fit&amp;w=1800 1200w,
                  https://infrequently.org/2024/01/performance-inequality-gap-2024/global_fixed_speedtest_medians.webp?nf_resize=fit&amp;w=1200   800w,
                  https://infrequently.org/2024/01/performance-inequality-gap-2024/global_fixed_speedtest_medians.webp?nf_resize=fit&amp;w=900   600w,
                  https://infrequently.org/2024/01/performance-inequality-gap-2024/global_fixed_speedtest_medians.webp?nf_resize=fit&amp;w=750   500w,
                  https://infrequently.org/2024/01/performance-inequality-gap-2024/global_fixed_speedtest_medians.webp?nf_resize=fit&amp;w=600   400w">
<img src="https://infrequently.org/2024/01/performance-inequality-gap-2024/global_fixed_speedtest_medians.webp" alt="Speedtest.net's global median values are unhelpful on their own, both because they represent users who are testing for speed (and not organic throughput) and because they don't give us a fuller understanding of the distribution." decoding="async" loading="lazy">
</source></picture>
</a>
  <figcaption>Speedtest.net's global median values are unhelpful on their own, both because they represent users who are testing for speed (and not organic throughput) and because they don't give us a fuller understanding of the distribution.</figcaption>
</figure>

<p>After many attempted Stupid Math Tricks with poorly fitting curves (bandwidth seems to be a funky cousin of log-normal), I've decided to wing it and beg for help: instead of trying to be clever, I'm leaning on <a href="https://radar.cloudflare.com/quality/">Cloudflare Radar's P25/P50/P75 distributions</a> for <a href="https://en.wikipedia.org/wiki/List_of_countries_by_number_of_Internet_users">populous, openly-connected countries with &gt;= ~50M internet users</a>. It's cheeky, but a weighted average of the P75 of download speeds (3/4ths of all connections are faster) should get us in the ballpark. We can then use the usual 5:1 downlink:uplink ratio to come up with an uplink estimate. We can also derive a weighted average for the P75 <abbr>RTT</abbr> from Cloudflare's data. Because Cloudflare doesn't distinguish mobile from desktop connections, this may be an overly conservative estimate, but it's still be more permissive than what we had been pegged to in years past:</p>

<table class="summary" id="natspeeds">
  <caption>National P75 Downlink and <abbr>RTT</abbr></caption>
  <thead>
    <tr>
      <td>Country</td>
      <td>P75 Downlink (Mbps)</td>
      <td>P75 <abbr>RTT</abbr> (ms)</td>
    </tr>
  </thead>
  <tbody>
    <tr>
      <td>India</td>
      <td>4</td>
      <td>114</td>
    </tr>
    <tr>
      <td>USA</td>
      <td>11</td>
      <td>58</td>
    </tr>
    <tr>
      <td>Indonesia</td>
      <td>5</td>
      <td>81</td>
    </tr>
    <tr>
      <td>Brazil</td>
      <td>8</td>
      <td>71</td>
    </tr>
    <tr>
      <td>Nigeria</td>
      <td>3</td>
      <td>201</td>
    </tr>
    <tr>
      <td>Pakistan</td>
      <td>3</td>
      <td>166</td>
    </tr>
    <tr>
      <td>Bangladesh</td>
      <td>5</td>
      <td>114</td>
    </tr>
    <tr>
      <td>Japan</td>
      <td>17</td>
      <td>42</td>
    </tr>
    <tr>
      <td>Mexico</td>
      <td>7</td>
      <td>75</td>
    </tr>
    <tr>
      <td>Egypt</td>
      <td>4</td>
      <td>100</td>
    </tr>
    <tr>
      <td>Germany</td>
      <td>16</td>
      <td>36</td>
    </tr>
    <tr>
      <td>Turkey</td>
      <td>7</td>
      <td>74</td>
    </tr>
    <tr>
      <td>Philippines</td>
      <td>7</td>
      <td>72</td>
    </tr>
    <tr>
      <td>Vietnam</td>
      <td>7</td>
      <td>72</td>
    </tr>
    <tr>
      <td>United Kingdom</td>
      <td>16</td>
      <td>37</td>
    </tr>
    <tr>
      <td>South Korea</td>
      <td>24</td>
      <td>26</td>
    </tr>
    <tr>
      <td><em>Weighted Avg.</em></td>
      <td>7.2</td>
      <td>94</td>
    </tr>
  </tbody>
</table>
<p>We, therefore, update our P75 link estimate <strong>7.2Mbps down, 1.4Mbps up, and 94ms <abbr>RTT</abbr>.</strong></p>
<p>This is a mild crime against statistics, not least of all because it averages unlike quantities and fails to sift mobile from desktop, but all the other methods available at time of writing are just as bad. Regardless, this new baseline is half again as much link capacity as last year, showing measurable improvement in networks worldwide.</p>
<p>If you or your company are able to generate a credible worldwide latency estimate in the higher percentiles for next year's update, please <a href="https://infrequently.org/about-me/">get in touch</a>.</p>
<h4 id="market-factors-1">Market Factors <a class="permalink" href="#market-factors-1">#</a></h4>
<p>The forces that shape the PC population have been largely fixed for many years. Since 2010, <a href="https://en.wikipedia.org/wiki/Market_share_of_personal_computer_vendors#Worldwide_(1996%E2%80%932022)">volumes have been on a slow downward glide path</a>, shrinking from ~350MM per year in a decade ago to ~260MM in 2018. The pandemic buying spree of 2021 pushed volumes above 300MM per year for the first time in eight years, with the vast majority of those devices being sold at low-end price points — think ~$300 Chromebooks rather than M1 MacBooks.</p>
<p>Lest we assume low-end means "short-lived", <a href="https://blog.google/outreach-initiatives/education/automatic-update-extension-chromebook/">recent announcements regarding software support for these devices</a> will considerably extend their impact. This low-end cohort will filter through the device population for years to come, pulling our performance budgets down, even as renewed process improvement is unlocking improved power efficiency and performance at the high end of the first-sale market. This won't be as pronounced as the diffusion of $100 smartphones has been in emerging markets, but the longer life-span of desktops is already a factor in our model.</p>
<h4 id="test-device-recommendations-1">Test Device Recommendations <a class="permalink" href="#test-device-recommendations-1">#</a></h4>
<p>Per our methodology from last year which uses the 5-8 year replacement cycle for a PC, we update our target date to late 2017 or early 2018, but leave the average-selling-price fixed between $600-700. Eventually we'll need to factor in the past couple of years of gyrations in inflation and supply chains into account when making an estimate, but not this year.</p>
<p>So what did $650, give or take, buy in late 2017 or early 2018?</p>
<p>One option was a <a href="https://www.theverge.com/2017/5/30/15698476/dell-inspiron-gaming-desktop-announced">naf looking tower from Dell, optimistically pitched at gamers</a>, with a <abbr>CPU</abbr> that scores <a href="https://browser.geekbench.com/v6/cpu/compare/3639070?baseline=253445">poorly versus a modern phone.</a>, but which blessedly sports 8GB of RAM.</p>
<p>In laptops (the larger segment), ~$650 bought the <a href="https://www.pcmag.com/reviews/lenovo-yoga-720-12-inch">Lenovo Yoga 720 (12")</a>, with a 2-core (4-thread) <a href="https://ark.intel.com/content/www/us/en/ark/products/95442/intel-core-i3-7100u-processor-3m-cache-2-40-ghz.html">Core i3-7100U</a> and 4GB of RAM. Versions with more RAM and a faster chip were available, but cost considerably more than our budget. This was not a fast box. <a href="https://browser.geekbench.com/v6/cpu/compare/3639070?baseline=4311168">Here's a device with that CPU compared to a modern phone</a>; not pretty:</p>
<figure>
<a href="https://browser.geekbench.com/v6/cpu/compare/3639070?baseline=4311168" alt="The phones of wealthy developers absolutely smoke the baseline PC." target="_new">
<picture class="preview">
  <source sizes="(max-width: 1200px) 70vw, 600px" srcset="https://infrequently.org/2024/01/performance-inequality-gap-2024/i3-7100U_vs_iphone_15_pro.webp?nf_resize=fit&amp;w=3600 2400w,
                  https://infrequently.org/2024/01/performance-inequality-gap-2024/i3-7100U_vs_iphone_15_pro.webp?nf_resize=fit&amp;w=2400 1600w,
                  https://infrequently.org/2024/01/performance-inequality-gap-2024/i3-7100U_vs_iphone_15_pro.webp?nf_resize=fit&amp;w=1800 1200w,
                  https://infrequently.org/2024/01/performance-inequality-gap-2024/i3-7100U_vs_iphone_15_pro.webp?nf_resize=fit&amp;w=1200   800w,
                  https://infrequently.org/2024/01/performance-inequality-gap-2024/i3-7100U_vs_iphone_15_pro.webp?nf_resize=fit&amp;w=900   600w,
                  https://infrequently.org/2024/01/performance-inequality-gap-2024/i3-7100U_vs_iphone_15_pro.webp?nf_resize=fit&amp;w=750   500w,
                  https://infrequently.org/2024/01/performance-inequality-gap-2024/i3-7100U_vs_iphone_15_pro.webp?nf_resize=fit&amp;w=600   400w">
<img src="https://infrequently.org/2024/01/performance-inequality-gap-2024/i3-7100U_vs_iphone_15_pro.webp" alt="The phones of wealthy developers absolutely smoke the baseline PC." decoding="async" loading="lazy">
</source></picture>
</a>
  <figcaption>The phones of wealthy developers absolutely smoke the baseline PC.</figcaption>
</figure>
<p>It's considerably faster than <a href="https://www.bhphotovideo.com/c/product/1704697-REG/hp_6k639ut_aba_probook_fortis_14_n4500.html">some devices still being sold to schools, though</a>.</p>
<p>What does this mean for our target devices? There's wild variation in performance per dollar below $600 which will only increase a inflation-affected cohorts grow to represent a larger fraction of the fleet. Intel's move (finally!) off of 14nm also means that gains are starting to arrive at the low end, but in an uneven way. General advice is therefore hard to issue. That said, we can triangulate based on what we know about the market:</p>

<p>My recommendation, then, to someone setting up a new lab today is not to spend more than $350 on new a test device. Consider laptops with chips like the <a href="https://www.intel.com/content/www/us/en/products/sku/197309/intel-celeron-processor-n4120-4m-cache-up-to-2-60-ghz/specifications.html">N4120</a>, <a href="https://www.intel.com/content/www/us/en/products/sku/212326/intel-celeron-processor-n4500-4m-cache-up-to-2-80-ghz/specifications.html">N4500</a>, or the <a href="https://www.intel.com/content/www/us/en/products/sku/212328/intel-celeron-processor-n5105-4m-cache-up-to-2-90-ghz/specifications.html">N5105</a>. Test devices should also have no more than 8GB of RAM, and preferably 4GB. The <a href="https://a.co/d/98wiGAl">2021 HP 14</a> is a fine proxy. The <a href="https://www.bhphotovideo.com/c/product/1761467-REG/hp_7f424ua_aba_14_14_ep0010nr_laptop_intel.html">updated ~$375 version</a> will do in a pinch, but try to spend less if you can. <a href="https://www.notebookcheck.net/Mobile-Processors-Benchmark-List.2436.0.html?type=&amp;sort=&amp;search=Intel&amp;itemselect_13189=13189&amp;itemselect_13111=13111&amp;itemselect_11533=11533&amp;itemselect_13079=13079&amp;or=0&amp;itemselect_13189=13189&amp;itemselect_13111=13111&amp;itemselect_11533=11533&amp;itemselect_13079=13079&amp;showCount=1&amp;showBars=1&amp;geekbench5_1_single=1&amp;geekbench5_1_multi=1&amp;geekbench6_2_single=1&amp;geekbench6_2_multi=1&amp;octane2=1&amp;speedometer=1&amp;cpu_fullname=1&amp;codename=1&amp;l2cache=1&amp;l3cache=1&amp;tdp=1&amp;mhz=1&amp;turbo_mhz=1&amp;cores=1&amp;threads=1">Test devices should preferably score no higher than 1,000 in single-core Geekbench 6 tests</a>; a line <a href="https://browser.geekbench.com/v6/cpu/4352217">the HP 14's N4120 easily ducks, clocking in at just over 350</a>.</p>
<h2 id="takeaways">Takeaways <a class="permalink" href="#takeaways">#</a></h2>
<p>There's a lot of good news embedded in this year's update. Devices and networks have finally started to get a bit faster (as predicted), pulling budgets upwards.</p>
<p>At the same time, the community remains in solid denial about the disastrous consequences of an over-reliance on JavaScript. This paints a picture of path dependence — front-end isn't moving on from approaches that hurt users, <a href="https://x.com/FredKSchott/status/1744842592905552227?s=20">even as the costs shift back onto teams that have been degrading life for users at the margins</a>.</p>
<p>We can anticipate continued improvement in devices over the next few years, and network pace may level out somewhat as the uneven deployment of 5G lurches forward. Regardless, the gap between the digital haves and have-nots continues to grow. Those least able to afford the fast devices are actively taxed by developers high on their own developer experience (<abbr title="Developer Experience">DX</abbr>).</p>
<p>It's not a mystery why folks who spend every waking hour inside a digital privilege bubble are not building with empathy or humility when nobody calls them to account. What's mysterious is that anybody pays them to do it. The Product Management (<abbr>PM</abbr>) and Engineering Management (<abbr>EM</abbr>) disciplines have utterly failed organisations building on the web, failing to put pro-user and pro-business constraints on the enthusiasms of developers.</p>
<p>Instead of cabining the the enthusiasms of the FP crowd, managers meekly repeated bullshit about how <em>"you can't hire for fundamentals"</em> as they waved in busloads of bootcampers whose React-heavy <abbr>CV</abbr> paint jobs had barely dried. They could have run bake-offs. They could have paid for skills that would serve the business over time. They could have facilitated learning anything the business valued. Instead, they abdicated. The kicker is that they didn't even reliably make things better for the class they imagined they were serving.</p>
<p>This post was partially drafted on airplane wifi, and I can assure you that wealthy folks also experience <abbr>RTT</abbr>'s north of 500ms and <a href="https://en.wikipedia.org/wiki/Gogo_Inflight_Internet#Technologies">channel capacity in the single-digit-Mbps</a>.</p>
<p>Even the wealthiest users step out of the privilege bubble sometimes. Are these EMs and PMs <em>really</em> happy to lose that business?</p>
<figure>
<a href="https://infrequently.org/2024/01/performance-inequality-gap-2024/airplane_wifi_speedtest.jpg" alt="&lt;em&gt;Tap for a larger version.&lt;/em&gt;&lt;br&gt;Wealthy users are going to experience networks with properties that are even worse than the 'bad' networks offered to the Next Billion Users. At an altitude of 40k feet and a ground speed for 580 MPH somewhere over Alberta, CA, your correspondent's bandwidth is scarce, lopsided, and laggy." target="_new">
<picture class="preview">
  <source sizes="(max-width: 1200px) 70vw, 600px" srcset="https://infrequently.org/2024/01/performance-inequality-gap-2024/airplane_wifi_speedtest.jpg?nf_resize=fit&amp;w=3600 2400w,
                  https://infrequently.org/2024/01/performance-inequality-gap-2024/airplane_wifi_speedtest.jpg?nf_resize=fit&amp;w=2400 1600w,
                  https://infrequently.org/2024/01/performance-inequality-gap-2024/airplane_wifi_speedtest.jpg?nf_resize=fit&amp;w=1800 1200w,
                  https://infrequently.org/2024/01/performance-inequality-gap-2024/airplane_wifi_speedtest.jpg?nf_resize=fit&amp;w=1200   800w,
                  https://infrequently.org/2024/01/performance-inequality-gap-2024/airplane_wifi_speedtest.jpg?nf_resize=fit&amp;w=900   600w,
                  https://infrequently.org/2024/01/performance-inequality-gap-2024/airplane_wifi_speedtest.jpg?nf_resize=fit&amp;w=750   500w,
                  https://infrequently.org/2024/01/performance-inequality-gap-2024/airplane_wifi_speedtest.jpg?nf_resize=fit&amp;w=600   400w">
<img src="https://infrequently.org/2024/01/performance-inequality-gap-2024/airplane_wifi_speedtest.jpg" alt="&lt;em&gt;Tap for a larger version.&lt;/em&gt;&lt;br&gt;Wealthy users are going to experience networks with properties that are even worse than the 'bad' networks offered to the Next Billion Users. At an altitude of 40k feet and a ground speed for 580 MPH somewhere over Alberta, CA, your correspondent's bandwidth is scarce, lopsided, and laggy." decoding="async" loading="lazy">
</source></picture>
</a>
  <figcaption><em>Tap for a larger version.</em><br>Wealthy users are going to experience networks with properties that are even worse than the 'bad' networks offered to the Next Billion Users. At an altitude of 40k feet and a ground speed for 580 MPH somewhere over Alberta, CA, your correspondent's bandwidth is scarce, lopsided, and laggy.</figcaption>
</figure>
<p>Of course, any trend that can't continue won't, and <abbr>INP</abbr>'s impact is already being felt. The great JavaScript merry-go-round may grind to a stop, but the momentum of consistently bad choices is formidable. Like passengers on a cruise ship ramming a boardwalk at flank speed, JavaScript regret is dawning far too late and interacting very poorly with something we ate. As the good ship Scripting shudders and lists on the remains of the Ferris Wheel, it's not exactly clear how to get off, but the choices that led us here are at least visible, if only by their belated consequences.</p>
<h3 id="the-great-branch-mispredict">The Great Branch Mispredict <a class="permalink" href="#the-great-branch-mispredict">#</a></h3>
<p>We got to a place where performance has been a constant problem in large part because a tribe of programmers convinced themselves that it <em>wasn't</em> and <em>wouldn't be</em>. The circa '13 narrative asserted that:</p>
<ul>
<li>CPUs would keep getting faster (just like they always had).</li>
<li>Networks would get better, or at least not get worse.</li>
<li>Organisations had all learned the lessons of Google and FaceBook's adventures in Ajax.</li>
</ul>
<p>It was all bullshit, <em>and many of us spotted it a mile away</em>.</p>
<p>But tribalism-boosted confirmation bias mixed with JavaScript's toxic positivity culture to precipitate out a Silicon Prosperity Gospel; all resources would go infinite if you just <em>believed</em>. No matter how wrong the premise, we kept executing down the obviously-falsified branch until the buffers drained.</p>
<p>The solutions are social, not technical, because the the delusions are social, rather than technical.</p>
<p>The stories that propped up <abbr>IE8</abbr>-focused frameworks like Angular and React in the mobile era have only served as comforting myths to ward off emerging device and network reality. For the past decade, the important question hasn't been if enough good technology existed, but rather how long the delusions would keep hold.</p>
<p>The community wanted to live in a different world than the one we inhabit, so we collectively mis-predicted. A healthy web community will value learning faster.</p>
<p>How deep was the branch? And how many cycles will the fault cost us? If CPUs and networks continue to improve at the rate of the past two years, and <abbr>INP</abbr> finally forces a reckoning, the answer might be as little as a decade. I fear we will not be so lucky; an entire generation has been trained to ignore reality, to prize tribalism rather than engineering rigor, and to devalue fundamentals. Those folks may not find the next couple of years to their liking.</p>
<p>Front-end's hangover from the JavaScript party is gonna <em>suck</em>.</p>