title: A formula for responsive font-size
url: https://jameshfisher.com/2024/03/12/a-formula-for-responsive-font-size/
hash_url: 08b2b2735f74ebf856d8a850cc9e41cc
archive_date: 2024-03-13
og_image: https://jameshfisher.com/assets/jim_512.jpg
description: This CSS is now part of most websites I make:
favicon: https://jameshfisher.com/assets/jim_128.png
language: en_US

<p>This CSS is now part of most websites I make:</p>
<pre><code class="language-css">:root {
  font-size: calc(1rem + 0.25vw);
}
</code></pre>
<p>This rule is an alternative to <code>@media</code> rules like this from nytimes.com:</p>
<pre><code class="language-css">p { font-size: 1.125rem; }
@media (min-width: 740px) {
p { font-size: 1.25rem; }
}
</code></pre>
<p>This pattern is the norm:
a small font size, a large font size, and a breakpoint.
Here’s a sample of common sites:</p>
<table>
<thead>
<tr>
<th></th>
<th>Small font-size</th>
<th>Large font-size</th>
<th>Breakpoint</th>
</tr>
</thead>
<tbody>
<tr>
<td><strong>Medium.com</strong></td>
<td><code>1.125rem</code></td>
<td><code>1.25rem</code></td>
<td><code>728px</code></td>
</tr>
<tr>
<td><strong>Substack.com</strong></td>
<td><code>1.125rem</code></td>
<td><code>1.25rem</code></td>
<td><code>768px</code></td>
</tr>
<tr>
<td><strong>Nytimes.com</strong></td>
<td><code>1.125rem</code></td>
<td><code>1.25rem</code></td>
<td><code>740px</code></td>
</tr>
</tbody>
</table>
<p>But breakpoints are mathematically ugly!
Instead of defining <code>font-size</code> piecewise,
can’t we use one linear function?
Here’s the line I believe they’re trying to approximate:</p>
<p><img src="/assets/2024-03-12/chart.png"></p>
<p>With modern CSS, we can just write that function!
It’s <code>calc(1.0625rem + 0.2604vw)</code>.
I round this to <code>1rem + 0.25vw</code>.</p>
<p>Sharp-eyed readers might wonder:
doesn’t my CSS have circular reasoning?
If <code>rem</code> is defined as “Font size of the root element”,
how can we use <code>1rem</code> in the definition of <code>font-size</code> on the root element?!
It turns out <a href="https://www.w3.org/TR/css-values-3/#rem">it’s a special case</a>:</p>
<blockquote>
<p>When specified in the <code>font-size</code> property of the root element,
or in a document with no root element,
<code>1rem</code> is equal to the initial value of the <code>font-size</code> property.</p>
</blockquote>
