title: RSS 2.0 Specification
url: https://www.rssboard.org/rss-specification#extendingRss
hash_url: 85b765a918ef094a5a2dd13a1ff5dd7d
archive_date: 2024-03-01
og_image: https://www.rssboard.org/images/left-arrow.gif
description: The current version of the RSS 2.0 specification (2.0.11), published on March 30, 2009.
favicon: https://www.rssboard.org/favicon.ico
language: en_US

<h3>RSS 2.0 Specification</h3>

<p class="fineprint">Editor's Note: This is the current version of the RSS 2.0 specification, published by the RSS Advisory Board on March 30, 2009 as version 2.0.11. The current version of the RSS spec will always be available at <a href="https://www.rssboard.org/rss-specification">this link</a>, all changes have been <a href="https://www.rssboard.org/rss-change-notes">logged</a> and <a href="https://www.rssboard.org/rss-history">other revisions</a> have been archived.</p>

<h4>Contents</h4>



<h4>What is RSS?<a name="whatIsRss"></a> <a href="#whatIsRss" title="What is RSS?"><img src="images/left-arrow.gif" border="0" alt="*"></a></h4>

<p>RSS is a Web content syndication format.</p>

<p>Its name is an acronym for <i><b>R</b>eally <b>S</b>imple <b>S</b>yndication.</i></p>

<p>RSS is a dialect of XML. All RSS files must conform to the XML 1.0 <a href="http://www.w3.org/TR/REC-xml">specification</a>, as published on the World Wide Web Consortium (W3C) website.</p>

<p>A summary of <a href="https://www.rssboard.org/rss-history">RSS version history</a>.</p>

<p>At the top level, a RSS document is a &lt;rss&gt; element, with a mandatory attribute called version, that specifies the version of RSS that the document conforms to. If it conforms to this specification, the version attribute must be 2.0.</p>

<p>Subordinate to the &lt;rss&gt; element is a single &lt;channel&gt; element, which contains information about the channel (metadata) and its contents.</p>

<h4>Sample files<a name="sampleFiles"></a> <a href="#sampleFiles" title="Sample files"><img src="images/left-arrow.gif" border="0" alt="*"></a></h4>

<p>Here are sample files for: RSS <a href="https://www.rssboard.org/files/sample-rss-091.xml">0.91</a>, <a href="https://www.rssboard.org/files/sample-rss-092.xml">0.92</a> and <a href="https://www.rssboard.org/files/sample-rss-2.xml">2.0</a>.</p>

<p>Note that the sample files may point to documents and services that no longer exist. The 0.91 sample was created when the 0.91 docs were written. Maintaining a trail of samples seems like a good idea.</p>

<h4>About this document<a name="aboutThisDocument"></a> <a href="#aboutThisDocument" title="About this document"><img src="images/left-arrow.gif" border="0" alt="*"></a></h4>

<p><img src="images/flowers.gif" border="0" align="right" hspace="15" vspace="5" alt="A bouquet of flowers, the symbol of RSS 2.0">This document represents the current status of RSS, incorporating all changes and additions starting with the basic spec for <a href="https://www.rssboard.org/rss-0-9-1">RSS 0.91</a> (June 2000) and follows <a href="https://www.rssboard.org/rss-0-9-2">RSS 0.92</a> (December 2000), <a href="https://www.rssboard.org/rss-2-0">RSS 2.0</a> (August 2002), and <a href="https://www.rssboard.org/rss-2-0-1">RSS 2.0.1</a> (July 2003). Change notes are <a href="https://www.rssboard.org/rss-change-notes">here</a>.</p>

<p>First we document the required and optional sub-elements of &lt;channel&gt;; and then document the sub-elements of &lt;item&gt;. The final sections answer frequently asked questions, and provide a roadmap for future evolution, and guidelines for extending RSS.</p>

<p>The <a href="https://www.rssboard.org/rss-profile">RSS Profile</a> contains a set of recommendations for how to create RSS documents that work best in the wide and diverse audience of client software that supports the format.</p>

<p>RSS documents can be tested for validity in the <a href="https://www.rssboard.org/rss-validator/">RSS Validator</a>.</p>

<h4>Required channel elements<a name="requiredChannelElements"></a> <a href="#requiredChannelElements" title="Required channel elements"><img src="images/left-arrow.gif" border="0" alt="*"></a></h4>

<p>Here's a list of the required channel elements, each with a brief description, an example, and where available, a pointer to a more complete description.</p>

<table class="table" cellspacing="10">
<tr valign="top"><td>Element</td><td>Description</td><td>Example</td></tr>
<tr valign="top"><td><span class="element">title</span></td><td>The name of the channel. It's how people refer to your service. If you have an HTML website that contains the same information as your RSS file, the title of your channel should be the same as the title of your website.</td><td class="examplecell">GoUpstate.com News Headlines</td></tr>
<tr valign="top"><td><span class="element">link</span></td><td>The URL to the HTML website corresponding to the channel.</td><td class="examplecell">http://www.goupstate.com/</td></tr>
<tr valign="top"><td><span class="element">description</span></td><td>Phrase or sentence describing the channel.</td><td class="examplecell">The latest news from GoUpstate.com, a Spartanburg Herald-Journal Web site.</td></tr>
</table>

<h4>Optional channel elements<a name="optionalChannelElements"></a> <a href="#optionalChannelElements" title="Optional channel elements"><img src="images/left-arrow.gif" border="0" alt="*"></a></h4>

<p>Here's a list of optional channel elements.</p>

<table class="table" cellspacing="10">
<tr valign="top"><td>Element</td><td>Description</td><td>Example</td></tr>
<tr valign="top"><td><span class="element">language</span></td><td>The language the channel is written in. This allows aggregators to group all Italian language sites, for example, on a single page. A list of allowable values for this element, as provided by Netscape, is <a href="https://www.rssboard.org/rss-language-codes">here</a>. You may also use <a href="http://www.w3.org/TR/REC-html40/struct/dirlang.html#langcodes">values defined</a> by the W3C.</td><td class="examplecell">en-us</td></tr>
<tr valign="top"><td><span class="element">copyright</span></td><td>Copyright notice for content in the channel.</td><td class="examplecell">Copyright 2002, Spartanburg Herald-Journal</td></tr>
<tr valign="top"><td><span class="element">managingEditor</span></td><td>Email address for person responsible for editorial content.</td><td class="examplecell">geo@herald.com (George Matesky)</td></tr>
<tr valign="top"><td><span class="element">webMaster</span></td><td>Email address for person responsible for technical issues relating to channel.</td><td class="examplecell">betty@herald.com (Betty Guernsey)</td></tr>
<tr valign="top"><td><span class="element">pubDate</span></td><td>The publication date for the content in the channel. For example, the <i>New York Times</i> publishes on a daily basis, the publication date flips once every 24 hours. That's when the pubDate of the channel changes. All date-times in RSS conform to the Date and Time Specification of <a href="http://asg.web.cmu.edu/rfc/rfc822.html">RFC 822</a>, with the exception that the year may be expressed with two characters or four characters (four preferred).</td><td class="examplecell">Sat, 07 Sep 2002 00:00:01 GMT</td></tr>
<tr valign="top"><td><span class="element">lastBuildDate</span></td><td>The last time the content of the channel changed.</td><td class="examplecell">Sat, 07 Sep 2002 09:42:31 GMT</td></tr>
<tr valign="top"><td><span class="element">category</span></td><td>Specify one or more categories that the channel belongs to. Follows the same rules as the &lt;item&gt;-level <a href="#ltcategorygtSubelementOfLtitemgt">category</a> element. More <a href="#syndic8">info</a>.</td><td class="examplecell">&lt;category&gt;Newspapers&lt;/category&gt;</td></tr>
<tr valign="top"><td><span class="element">generator</span></td><td>A string indicating the program used to generate the channel.</td><td class="examplecell">MightyInHouse Content System v2.3</td></tr>
<tr valign="top"><td><span class="element">docs</span></td><td>A URL that points to the <a href="https://www.rssboard.org/rss-specification">documentation</a> for the format used in the RSS file. It's probably a pointer to this page. It's for people who might stumble across an RSS file on a Web server 25 years from now and wonder what it is.</td><td class="examplecell">https://www.rssboard.org/rss-specification</td></tr>
<tr valign="top"><td><span class="element">cloud</span></td><td>Allows processes to register with a cloud to be notified of updates to the channel, implementing a lightweight publish-subscribe protocol for RSS feeds. More info <a href="#ltcloudgtSubelementOfLtchannelgt">here</a>.</td><td class="examplecell">&lt;cloud domain="rpc.sys.com" port="80" path="/RPC2" registerProcedure="pingMe" protocol="soap"/&gt;</td></tr>
<tr valign="top"><td><span class="element">ttl</span></td><td>ttl stands for time to live. It's a number of minutes that indicates how long a channel can be cached before refreshing from the source. More info <a href="#ltttlgtSubelementOfLtchannelgt">here</a>.</td><td class="examplecell">&lt;ttl&gt;60&lt;/ttl&gt;</td></tr>
<tr valign="top"><td><span class="element">image</span></td><td>Specifies a GIF, JPEG or PNG image that can be displayed with the channel. More info <a href="#ltimagegtSubelementOfLtchannelgt">here</a>.</td><td></td></tr>
<tr valign="top"><td><span class="element">rating</span></td><td>The <a href="http://www.w3.org/PICS/">PICS</a> rating for the channel.</td><td></td></tr>
<tr valign="top"><td><span class="element">textInput</span></td><td>Specifies a text input box that can be displayed with the channel. More info <a href="#lttextinputgtSubelementOfLtchannelgt">here</a>.</td><td></td></tr>
<tr valign="top"><td><span class="element">skipHours</span></td><td>A hint for aggregators telling them which hours they can skip. This element contains up to 24 &lt;hour&gt; sub-elements whose value is a number between 0 and 23, representing a time in GMT, when aggregators, if they support the feature, may not read the channel on hours listed in the &lt;skipHours&gt; element. The hour beginning at midnight is hour zero.</td><td></td></tr>
<tr valign="top"><td><span class="element">skipDays</span></td><td>A hint for aggregators telling them which days they can skip. This element contains up to seven &lt;day&gt; sub-elements whose value is Monday, Tuesday, Wednesday, Thursday, Friday, Saturday or Sunday. Aggregators may not read the channel during days listed in the &lt;skipDays&gt; element.</td><td></td></tr>
</table>

<h5>&lt;image&gt; sub-element of &lt;channel&gt;<a name="ltimagegtSubelementOfLtchannelgt"></a> <a href="#ltimagegtSubelementOfLtchannelgt" title="&lt;image&gt; sub-element of &lt;channel&gt;"><img src="images/left-arrow.gif" border="0" alt="*"></a></h5>

<p>&lt;image&gt; is an optional sub-element of &lt;channel&gt;, which contains three required and three optional sub-elements.</p>

<p>&lt;url&gt; is the URL of a GIF, JPEG or PNG image that represents the channel.</p>

<p>&lt;title&gt; describes the image, it's used in the ALT attribute of the HTML &lt;img&gt; tag when the channel is rendered in HTML.</p>

<p>&lt;link&gt; is the URL of the site, when the channel is rendered, the image is a link to the site. (Note, in practice the image &lt;title&gt; and &lt;link&gt; should have the same value as the channel's &lt;title&gt; and &lt;link&gt;.</p>

<p>Optional elements include &lt;width&gt; and &lt;height&gt;, numbers, indicating the width and height of the image in pixels. &lt;description&gt; contains text that is included in the TITLE attribute of the link formed around the image in the HTML rendering.</p>

<p>Maximum value for width is 144, default value is 88.</p>

<p>Maximum value for height is 400, default value is 31.</p>

<h5>&lt;cloud&gt; sub-element of &lt;channel&gt;<a name="ltcloudgtSubelementOfLtchannelgt"></a> <a href="#ltcloudgtSubelementOfLtchannelgt" title="&lt;cloud&gt; sub-element of &lt;channel&gt;"><img src="images/left-arrow.gif" border="0" alt="*"></a></h5>

<p>&lt;cloud&gt; is an optional sub-element of &lt;channel&gt;.</p>

<p>It specifies a web service that supports the rssCloud interface which can be implemented in HTTP-POST, XML-RPC or SOAP 1.1.</p>

<p>Its purpose is to allow processes to register with a cloud to be notified of updates to the channel, implementing a lightweight publish-subscribe protocol for RSS feeds.</p>

<p class="example">&lt;cloud domain="rpc.sys.com" port="80" path="/RPC2" registerProcedure="myCloud.rssPleaseNotify" protocol="xml-rpc" /&gt;</p>

<p>In this example, to request notification on the channel it appears in, you would send an XML-RPC message to rpc.sys.com on port 80, with a path of /RPC2. The procedure to call is myCloud.rssPleaseNotify.</p>

<p>A full explanation of this element and the rssCloud interface is <a href="https://www.rssboard.org/rsscloud-interface">here</a>.</p>

<h5>&lt;ttl&gt; sub-element of &lt;channel&gt;<a name="ltttlgtSubelementOfLtchannelgt"></a> <a href="#ltttlgtSubelementOfLtchannelgt" title="&lt;ttl&gt; sub-element of &lt;channel&gt;"><img src="images/left-arrow.gif" border="0" alt="*"></a></h5>

<p>&lt;ttl&gt; is an optional sub-element of &lt;channel&gt;.</p>

<p>ttl stands for time to live. It's a number of minutes that indicates how long a channel can be cached before refreshing from the source. This makes it possible for RSS sources to be managed by a file-sharing network such as Gnutella.</p>

<p>Example:</p>

<p class="example">&lt;ttl&gt;60&lt;/ttl&gt;</p>

<h5>&lt;textInput&gt; sub-element of &lt;channel&gt;<a name="lttextinputgtSubelementOfLtchannelgt"></a> <a href="#lttextinputgtSubelementOfLtchannelgt" title="&lt;textInput&gt; sub-element of &lt;channel&gt;"><img src="images/left-arrow.gif" border="0" alt="*"></a></h5>

<p>A channel may optionally contain a &lt;textInput&gt; sub-element, which contains four required sub-elements.</p>

<p>&lt;title&gt; -- The label of the Submit button in the text input area.</p>

<p>&lt;description&gt; -- Explains the text input area.</p>

<p>&lt;name&gt; -- The name of the text object in the text input area.</p>

<p>&lt;link&gt; -- The URL of the CGI script that processes text input requests.</p>

<p>The purpose of the &lt;textInput&gt; element is something of a mystery. You can use it to specify a search engine box. Or to allow a reader to provide feedback. Most aggregators ignore it.</p>

<h4>Elements of &lt;item&gt;<a name="hrelementsOfLtitemgt"></a> <a href="#hrelementsOfLtitemgt" title="Elements of &lt;item&gt;"><img src="images/left-arrow.gif" border="0" alt="*"></a></h4>

<p>A channel may contain any number of &lt;item&gt;s. An item may represent a "story" -- much like a story in a newspaper or magazine; if so its description is a synopsis of the story, and the link points to the full story. An item may also be complete in itself, if so, the description contains the text (entity-encoded HTML is allowed; see <a href="https://www.rssboard.org/rss-encoding-examples">examples</a>), and the link and title may be omitted. All elements of an item are optional, however at least one of title or description must be present.</p>

<table class="table" cellspacing="10">
<tr valign="top"><td>Element</td><td>Description</td><td>Example</td></tr>
<tr valign="top"><td><span class="element">title</span></td><td>The title of the item.</td><td class="examplecell">Venice Film Festival Tries to Quit Sinking</td></tr>
<tr valign="top"><td><span class="element">link</span></td><td>The URL of the item.</td><td class="examplecell">http://nytimes.com/2004/12/07FEST.html</td></tr>
<tr valign="top"><td><span class="element">description</span></td><td>The item synopsis.</td><td class="examplecell">&lt;description&gt;Some of the most heated chatter at the Venice Film Festival this week was about the way that the arrival of the stars at the Palazzo del Cinema was being staged.&lt;/description&gt;</td></tr>
<tr valign="top"><td><span class="element">author</span></td><td>Email address of the author of the item. <a href="#ltauthorgtSubelementOfLtitemgt">More</a>.</td><td></td></tr>
<tr valign="top"><td><span class="element">category</span></td><td>Includes the item in one or more categories. <a href="#ltcategorygtSubelementOfLtitemgt">More</a>.</td><td></td></tr>
<tr valign="top"><td><span class="element">comments</span></td><td>URL of a page for comments relating to the item. <a href="#ltcommentsgtSubelementOfLtitemgt">More</a>.</td><td></td></tr>
<tr valign="top"><td><span class="element">enclosure</span></td><td>Describes a media object that is attached to the item. <a href="#ltenclosuregtSubelementOfLtitemgt">More</a>.</td><td></td></tr>
<tr valign="top"><td><span class="element">guid</span></td><td>A string that uniquely identifies the item. <a href="#ltguidgtSubelementOfLtitemgt">More</a>.</td><td></td></tr>
<tr valign="top"><td><span class="element">pubDate</span></td><td>Indicates when the item was published. <a href="#ltpubdategtSubelementOfLtitemgt">More</a>.</td><td></td></tr>
<tr valign="top"><td><span class="element">source</span></td><td>The RSS channel that the item came from. <a href="#ltsourcegtSubelementOfLtitemgt">More</a>.</td><td></td></tr>
</table>

<h5>&lt;source&gt; sub-element of &lt;item&gt;<a name="ltsourcegtSubelementOfLtitemgt"></a> <a href="#ltsourcegtSubelementOfLtitemgt" title="&lt;source&gt; sub-element of &lt;item&gt;"><img src="images/left-arrow.gif" border="0" alt="*"></a></h5>

<p>&lt;source&gt; is an optional sub-element of &lt;item&gt;.</p>

<p>Its value is the name of the RSS channel that the item came from, derived from its &lt;title&gt;. It has one required attribute, url, which links to the XMLization of the source.</p>

<p class="example">&lt;source url="http://www.tomalak.org/links2.xml"&gt;Tomalak's Realm&lt;/source&gt;</p>

<p>The purpose of this element is to propagate credit for links, to publicize the sources of news items. It can be used in the Post command of an aggregator. It should be generated automatically when forwarding an item from an aggregator to a weblog authoring tool.</p>

<h5>&lt;enclosure&gt; sub-element of &lt;item&gt;<a name="ltenclosuregtSubelementOfLtitemgt"></a> <a href="#ltenclosuregtSubelementOfLtitemgt" title="&lt;enclosure&gt; sub-element of &lt;item&gt;"><img src="images/left-arrow.gif" border="0" alt="*"></a></h5>

<p>&lt;enclosure&gt; is an optional sub-element of &lt;item&gt;.</p>

<p>It has three required attributes. url says where the enclosure is located, length says how big it is in bytes, and type says what its type is, a standard MIME type.</p>

<p>The url must be an http url.</p>

<p class="example">&lt;enclosure url="http://www.scripting.com/mp3s/weatherReportSuite.mp3" length="12216320" type="audio/mpeg" /&gt;</p>

<p>A use-case narrative for this element is <a href="https://www.rssboard.org/rss-enclosures-use-case">here</a>.</p>

<h5>&lt;category&gt; sub-element of &lt;item&gt;<a name="ltcategorygtSubelementOfLtitemgt"></a> <a href="#ltcategorygtSubelementOfLtitemgt" title="&lt;category&gt; sub-element of &lt;item&gt;"><img src="images/left-arrow.gif" border="0" alt="*"></a></h5>

<p>&lt;category&gt; is an optional sub-element of &lt;item&gt;.</p>

<p>It has one optional attribute, domain, a string that identifies a categorization taxonomy.</p>

<p>The value of the element is a forward-slash-separated string that identifies a hierarchic location in the indicated taxonomy. Processors may establish conventions for the interpretation of categories. Two examples are provided below:</p>

<p class="example">&lt;category&gt;Grateful Dead&lt;/category&gt;</p>

<p class="example">&lt;category domain="http://www.fool.com/cusips"&gt;MSFT&lt;/category&gt;</p>

<p>You may include as many category elements as you need to, for different domains, and to have an item cross-referenced in different parts of the same domain.</p>

<h5>&lt;pubDate&gt; sub-element of &lt;item&gt;<a name="ltpubdategtSubelementOfLtitemgt"></a> <a href="#ltpubdategtSubelementOfLtitemgt" title="&lt;pubDate&gt; sub-element of &lt;item&gt;"><img src="images/left-arrow.gif" border="0" alt="*"></a></h5>

<p>&lt;pubDate&gt; is an optional sub-element of &lt;item&gt;.</p>

<p>Its value is a <a href="http://asg.web.cmu.edu/rfc/rfc822.html">date</a>, indicating when the item was published. If it's a date in the future, aggregators may choose to not display the item until that date.</p>

<p class="example">&lt;pubDate&gt;Sun, 19 May 2002 15:21:36 GMT&lt;/pubDate&gt;</p>

<h5>&lt;guid&gt; sub-element of &lt;item&gt;<a name="ltguidgtSubelementOfLtitemgt"></a> <a href="#ltguidgtSubelementOfLtitemgt" title="&lt;guid&gt; sub-element of &lt;item&gt;"><img src="images/left-arrow.gif" border="0" alt="*"></a></h5>

<p>&lt;guid&gt; is an optional sub-element of &lt;item&gt;.</p>

<p>guid stands for globally unique identifier. It's a string that uniquely identifies the item. When present, an aggregator may choose to use this string to determine if an item is new.</p>

<p class="example">&lt;guid&gt;http://some.server.com/weblogItem3207&lt;/guid&gt;</p>

<p>There are no rules for the syntax of a guid. Aggregators must view them as a string. It's up to the source of the feed to establish the uniqueness of the string.</p>

<p>If the guid element has an attribute named isPermaLink with a value of true, the reader may assume that it is a permalink to the item, that is, a url that can be opened in a Web browser, that points to the full item described by the &lt;item&gt; element.  An example:</p>

<p class="example">&lt;guid isPermaLink="true"&gt;http://inessential.com/2002/09/01.php#a2&lt;/guid&gt;</p>

<p>isPermaLink is optional, its default value is true. If its value is false, the guid may not be assumed to be a url, or a url to anything in particular.</p>

<h5>&lt;comments&gt; sub-element of &lt;item&gt;<a name="ltcommentsgtSubelementOfLtitemgt"></a> <a href="#ltcommentsgtSubelementOfLtitemgt" title="&lt;comments&gt; sub-element of &lt;item&gt;"><img src="images/left-arrow.gif" border="0" alt="*"></a></h5>

<p>&lt;comments&gt; is an optional sub-element of &lt;item&gt;.</p>

<p>If present, it is the url of the comments page for the item.</p>

<p class="example">&lt;comments&gt;http://ekzemplo.com/entry/4403/comments&lt;/comments&gt;</p>

<p>More about comments <a href="https://www.rssboard.org/rss-weblog-comments-use-case">here</a>.</p>

<h5>&lt;author&gt; sub-element of &lt;item&gt;<a name="ltauthorgtSubelementOfLtitemgt"></a> <a href="#ltauthorgtSubelementOfLtitemgt" title="&lt;author&gt; sub-element of &lt;item&gt;"><img src="images/left-arrow.gif" border="0" alt="*"></a></h5>

<p>&lt;author&gt; is an optional sub-element of &lt;item&gt;.</p>

<p>It's the email address of the author of the item. For newspapers and magazines syndicating via RSS, the author is the person who wrote the article that the &lt;item&gt; describes. For collaborative weblogs, the author of the item might be different from the managing editor or webmaster. For a weblog authored by a single individual it would make sense to omit the &lt;author&gt; element.</p>

<p class="example">&lt;author&gt;lawyer@boyer.net (Lawyer Boyer)&lt;/author&gt;</p>

<h4>Comments<a name="comments"></a> <a href="#comments" title="Comments"><img src="images/left-arrow.gif" border="0" alt="*"></a></h4>

<p>RSS places restrictions on the first non-whitespace characters of the data in &lt;link&gt; and &lt;url&gt; elements. The data in these elements must begin with an <a href="http://www.iana.org/assignments/uri-schemes">IANA-registered</a> URI scheme, such as http://, https://, news://, mailto: and ftp://. Prior to RSS 2.0, the specification only allowed http:// and ftp://, however, in practice other URI schemes were in use by content developers and supported by aggregators. Aggregators may have limits on the URI schemes they support. Content developers should not assume that all aggregators support all schemes.</p>

<p>In RSS 0.91, various elements are restricted to 500 or 100 characters. There can be no more than 15 &lt;items&gt; in a 0.91 &lt;channel&gt;. There are no string-length or XML-level limits in RSS 0.92 and greater. Processors may impose their own limits, and generators may have preferences that say no more than a certain number of &lt;item&gt;s can appear in a channel, or that strings are limited in length.</p>

<p>In RSS 2.0, <a name="syndic8"></a>a provision is made for linking a channel to its identifier in a cataloging system, using the channel-level category feature, described above. For example, to link a channel to its Syndic8 identifier, include a category element as a sub-element of &lt;channel&gt;, with domain "Syndic8", and value the identifier for your channel in the Syndic8 database. The appropriate category element for Scripting News would be &lt;category domain="Syndic8"&gt;1765&lt;/category&gt;.</p>

<p>A frequently asked question about &lt;guid&gt;s is how do they compare to &lt;link&gt;s. Aren't they the same thing? Yes, in some content systems, and no in others. In some systems, &lt;link&gt; is a permalink to a weblog item. However, in other systems, each &lt;item&gt; is a synopsis of a longer article, &lt;link&gt; points to the article, and &lt;guid&gt; is the permalink to the weblog entry. In all cases, it's recommended that you provide the guid, and if possible make it a permalink. This enables aggregators to not repeat items, even if there have been editing changes.</p>

<p>If you have questions about the RSS 2.0 format, please post them on the <a href="http://groups.yahoo.com/group/rss-public">RSS-Public</a> mailing list. The list, maintained by the RSS Advisory Board, serves as a support resource for users, authors and developers who are creating and using content in the format.</p>

<h4>Extending RSS<a name="extendingRss"></a> <a href="#extendingRss" title="Extending RSS"><img src="images/left-arrow.gif" border="0" alt="*"></a></h4>

<p>RSS originated in 1999, and has strived to be a simple, easy to understand format, with relatively modest goals. After it became a popular format, developers wanted to extend it using modules defined in namespaces, as <a href="http://www.w3.org/TR/REC-xml-names/">specified</a> by the W3C.</p>

<p>RSS 2.0 adds that capability, following a simple rule. A RSS feed may contain elements and attributes not described on this page, only if those elements and attributes are defined in a namespace.</p>

<p>The elements defined in this document are not themselves members of a namespace, so that RSS 2.0 can remain compatible with previous versions in the following sense -- a version 0.91 or 0.92 file is also a valid 2.0 file. If the elements of RSS 2.0 were in a namespace, this constraint would break, a version 0.9x file <i>would not</i> be a valid 2.0 file.</p>

<h4>Roadmap<a name="roadmap"></a> <a href="#roadmap" title="Roadmap"><img src="images/left-arrow.gif" border="0" alt="*"></a></h4>

<p>RSS is by no means a perfect format, but it is very popular and widely supported. Having a settled spec is something RSS has needed for a long time. The purpose of this work is to help it become a unchanging thing, to foster growth in the market that is developing around it, and to clear the path for innovation in new syndication formats. Therefore, the RSS spec is, for all practical purposes, frozen at version 2.0.1. We anticipate possible 2.0.2 or 2.0.3 versions, etc. only for the purpose of clarifying the specification, not for adding new features to the format. Subsequent work should happen in modules, using namespaces, and in completely new syndication formats, with new names.</p>

<h4>License and authorship<a name="licenseAndAuthorship"></a> <a href="#licenseAndAuthorship" title="License and authorship"><img src="images/left-arrow.gif" border="0" alt="*"></a></h4>
